@@ -110,6 +110,20 @@ public interface KeyValueStore<K, V> {
    */
   KeyValueIterator<K, V> range(K from, K to);
 
+  /**
+   * Returns a snapshot of this store for a sorted range of entries specified by [{@code from}, {@code to}).
+   * The snapshot is immutable - ie., any mutations to the store are not reflected in the snapshot after it is created.
+   *
+   * <p><b>API Note:</b> The returned snapshot MUST be closed after use.
+   * @param from the key specifying the low endpoint (inclusive) of the keys in the returned range.
+   * @param to the key specifying the high endpoint (exclusive) of the keys in the returned range.
+   * @return a snapshot for the specified key range.
+   * @throws NullPointerException if null is used for {@code from} or {@code to}.
+   */
+  default KeyValueSnapshot<K, V> snapshot(K from, K to) {
+    throw new UnsupportedOperationException("snapshot() is not supported in " + this.getClass().getName());
+  }
+
   /**
    * Returns an iterator for all entries in this key-value store.
    *
