@@ -26,6 +26,8 @@ import org.apache.samza.config.Config;
 import org.apache.samza.config.MapConfig;
 import org.apache.samza.config.StreamConfig;
 import org.apache.samza.system.eventhub.producer.EventHubSystemProducer;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import scala.collection.JavaConversions;
 
 import java.time.Duration;
@@ -77,21 +79,42 @@ public class EventHubConfig extends MapConfig {
   private static final int MESSAGE_HEADER_OVERHEAD = 24 * 1024;
   private static final int DEFAULT_MAX_MESSAGE_SIZE = 1024 * 1024 - MESSAGE_HEADER_OVERHEAD;
 
-  // Each EventHub client maintains single TCP connection. To improve throughput, we will instantiate one
-  // client for each partition. Allow the option to disable the feature in case too many EventHub clients
+  // Each Event Hubs client maintains single TCP connection. To improve throughput, we will instantiate one
+  // client for each partition. Allow the option to disable the feature in case too many Event Hubs clients
   // end up causing unpredictable issues when number of partitions is really high.
   public static final String CONFIG_PER_PARTITION_CONNECTION = "systems.%s.eventhubs.perPartition.connection";
   public static final Boolean DEFAULT_CONFIG_PER_PARTITION_CONNECTION = true;
 
+  /*
+   * This set of configs control the max retry count allowed within a certain sliding window, as well as
+   * the minimum interval between two retries.
+   * For example, if max retry count is 10, window size is 1 day, min retry interval is 10 min, then
+   * we retry up to 10 times within 1 day time frame and we only retry 10 min after the last retry.
+   */
+  public static final String CONFIG_MAX_RETRY_COUNT = "systems.%s.eventhubs.max.retry.count";
+  public static final long DEFAULT_CONFIG_MAX_RETRIES_COUNT = 3;
+  public static final String CONFIG_RETRY_WINDOW_MS = "systems.%s.eventhubs.retry.window.ms";
+  public static final long DEFAULT_CONFIG_RETRY_WINDOW_MS = Duration.ofHours(3).toMillis();
+  public static final String CONFIG_MIN_RETRY_INTERVAL_MS = "systems.%s.eventhubs.min.retry.interval.ms";
+  public static final long DEFAULT_CONFIG_RETRY_INTERVAL_MS = Duration.ofMinutes(3).toMillis();
+
   private final Map<String, String> physcialToId = new HashMap<>();
 
+  private static final Logger LOG = LoggerFactory.getLogger(EventHubConfig.class);
+
   public EventHubConfig(Config config) {
     super(config);
 
     // Build reverse index for streamName -> streamId
     StreamConfig streamConfig = new StreamConfig(config);
+
+    LOG.info("Building mappings from physicalName to streamId");
     JavaConversions.asJavaCollection(streamConfig.getStreamIds())
-            .forEach((streamId) -> physcialToId.put(streamConfig.getPhysicalName(streamId), streamId));
+        .forEach((streamId) -> {
+            String physicalName = streamConfig.getPhysicalName(streamId);
+            LOG.info("Obtained physicalName: {} for streamId: {} ", physicalName, streamId);
+            physcialToId.put(physicalName, streamId);
+          });
   }
 
   private String getFromStreamIdOrName(String configName, String streamName, String defaultString) {
@@ -128,7 +151,7 @@ public class EventHubConfig extends MapConfig {
 
   /**
    * Get the list of streams that are defined. Each stream has enough
-   * information for connecting to a certain EventHub entity.
+   * information for connecting to a certain Event Hubs entity.
    *
    * @param systemName name of the system
    * @return list of stream names
@@ -138,25 +161,27 @@ public class EventHubConfig extends MapConfig {
   }
 
   /**
-   * Get the EventHubs namespace for the stream
+   * Get the Event Hubs namespace for the stream
    *
    * @param systemName name of the system
    * @param streamName name of stream (physical or streamId)
-   * @return EventHubs namespace
+   * @return Event Hubs namespace
    */
   public String getStreamNamespace(String systemName, String streamName) {
+    LOG.info("Obtaining name-space for system: {} physical name: {}", systemName, streamName);
     return validateRequiredConfig(getFromStreamIdOrName(CONFIG_STREAM_NAMESPACE, streamName),
             "Namespace", systemName, streamName);
   }
 
   /**
-   * Get the EventHubs entity path (topic name) for the stream
+   * Get the Event Hubs entity path (topic name) for the stream
    *
    * @param systemName name of the system
    * @param streamName name of stream (physical or streamId)
-   * @return EventHubs entity path
+   * @return Event Hubs entity path
    */
   public String getStreamEntityPath(String systemName, String streamName) {
+    LOG.info("Obtaining entity-path for system: {} physical name: {}", systemName, streamName);
     return validateRequiredConfig(getFromStreamIdOrName(CONFIG_STREAM_ENTITYPATH, streamName),
             "EntityPath", systemName, streamName);
   }
@@ -181,16 +206,16 @@ public class EventHubConfig extends MapConfig {
   }
 
   /**
-   * Get the per partition prefetch count for the event hub client
+   * Get the per partition prefetch count for the Event Hubs client
    * @param systemName Name of the system.
-   * @return Per partition Prefetch count for the event hub client.
+   * @return Per partition Prefetch count for the Event Hubs client.
    */
   public Integer getPrefetchCount(String systemName) {
     return getInt(String.format(CONFIG_PREFETCH_COUNT, systemName), DEFAULT_CONFIG_PREFETCH_COUNT);
   }
 
   /**
-   * Get the EventHubs max Message size
+   * Get the Event Hubs max Message size
    *
    * @param systemName name of the system
    * @return the max message size supported in event hubs.
@@ -200,11 +225,11 @@ public class EventHubConfig extends MapConfig {
   }
 
   /**
-   * Get the EventHubs SAS (Shared Access Signature) key name for the stream
+   * Get the Event Hubs SAS (Shared Access Signature) key name for the stream
    *
    * @param systemName name of the system
    * @param streamName name of stream (physical or streamId)
-   * @return EventHubs SAS key name
+   * @return Event Hubs SAS key name
    */
   public String getStreamSasKeyName(String systemName, String streamName) {
     return validateRequiredConfig(getFromStreamIdOrName(CONFIG_STREAM_SAS_KEY_NAME, streamName),
@@ -212,11 +237,11 @@ public class EventHubConfig extends MapConfig {
   }
 
   /**
-   * Get the EventHubs SAS (Shared Access Signature) token for the stream
+   * Get the Event Hubs SAS (Shared Access Signature) token for the stream
    *
    * @param systemName name of the system
    * @param streamName name of stream (physical or streamId)
-   * @return EventHubs SAS token
+   * @return Event Hubs SAS token
    */
   public String getStreamSasToken(String systemName, String streamName) {
     return validateRequiredConfig(getFromStreamIdOrName(CONFIG_STREAM_SAS_TOKEN, streamName),
@@ -224,18 +249,18 @@ public class EventHubConfig extends MapConfig {
   }
 
   /**
-   * Get the EventHubs consumer group used for consumption for the stream
+   * Get the Event Hubs consumer group used for consumption for the stream
    *
    * @param systemName name of the system
    * @param streamName name of stream (physical or streamId)
-   * @return EventHubs consumer group
+   * @return Event Hubs consumer group
    */
   public String getStreamConsumerGroup(String systemName, String streamName) {
     return getFromStreamIdOrName(CONFIG_STREAM_CONSUMER_GROUP, streamName, DEFAULT_CONFIG_STREAM_CONSUMER_GROUP);
   }
 
   /**
-   * Get the partition method of the systemName. By default partitioning is handed by EventHub.
+   * Get the partition method of the systemName. By default partitioning is handled by Event Hubs.
    *
    * @param systemName name of the system
    * @return The method the producer should use to partition the outgoing data
@@ -262,7 +287,7 @@ public class EventHubConfig extends MapConfig {
   }
 
   /**
-   * Get the timeout for the getRuntimeInfo request to EventHub client
+   * Get the timeout for the getRuntimeInfo request to Event Hubs client
    *
    * @param systemName name of the systems
    * @return long, timeout in millis for fetching RuntimeInfo
@@ -273,7 +298,7 @@ public class EventHubConfig extends MapConfig {
   }
 
   /**
-   * Get the capacity of the Event Hub consumer buffer - the blocking queue used for storing messages
+   * Get the capacity of the Event Hubs consumer buffer - the blocking queue used for storing messages
    *
    * @param systemName name of the system
    * @return int, number of buffered messages per SystemStreamPartition
@@ -287,9 +312,12 @@ public class EventHubConfig extends MapConfig {
   }
 
   /**
-   * Returns whether to create one EventHub client per partition. Each EventHub client maintains
-   * single TCP connection. More EventHub clients will improve throughput in general.
+   * Returns whether to create one Event Hubs client per partition. Each Event Hubs client maintains
+   * single TCP connection. More Event Hubs clients will improve throughput in general.
    * For producer this config is only relevant when partition method is PARTITION_KEY_AS_PARTITION
+   *
+   * @param systemName name of the system
+   * @return true if an Event Hubs client should be created per partition, false otherwise
    */
   public Boolean getPerPartitionConnection(String systemName) {
     String isPerPartitionConnection = get(String.format(CONFIG_PER_PARTITION_CONNECTION, systemName));
@@ -298,4 +326,31 @@ public class EventHubConfig extends MapConfig {
     }
     return Boolean.valueOf(isPerPartitionConnection);
   }
+
+  /**
+   * Get max retry count allowed before propagating the exception to users
+   * @param systemaName name of the system
+   * @return long, max retry count allowed
+   */
+  public long getMaxRetryCount(String systemaName) {
+    return getLong(String.format(CONFIG_MAX_RETRY_COUNT, systemaName), DEFAULT_CONFIG_MAX_RETRIES_COUNT);
+  }
+
+  /**
+   * Get the sliding window size in ms for tracking the retry count
+   * @param systemName name of the system
+   * @return long, sliding window size in ms
+   */
+  public long getRetryWindowMs(String systemName) {
+    return getLong(String.format(CONFIG_RETRY_WINDOW_MS, systemName), DEFAULT_CONFIG_RETRY_WINDOW_MS);
+  }
+
+  /**
+   * Get the minimum interval in ms between two retries on non transient error
+   * @param systemName name of the system
+   * @return long, minimum interval in ms between retries
+   */
+  public long getMinRetryIntervalMs(String systemName) {
+    return getLong(String.format(CONFIG_MIN_RETRY_INTERVAL_MS, systemName), DEFAULT_CONFIG_RETRY_INTERVAL_MS);
+  }
 }
