@@ -20,6 +20,7 @@
 package org.apache.samza.system.eventhub.consumer;
 
 import com.google.common.annotations.VisibleForTesting;
+import com.google.common.util.concurrent.ThreadFactoryBuilder;
 import com.microsoft.azure.eventhubs.EventData;
 import com.microsoft.azure.eventhubs.EventHubException;
 import com.microsoft.azure.eventhubs.EventPosition;
@@ -28,17 +29,16 @@ import com.microsoft.azure.eventhubs.PartitionReceiver;
 import com.microsoft.azure.eventhubs.impl.ClientConstants;
 import java.time.Duration;
 import java.time.Instant;
-import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.BlockingQueue;
-import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.ExecutionException;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
+import java.util.concurrent.Future;
 import java.util.concurrent.LinkedBlockingQueue;
 import java.util.concurrent.TimeUnit;
-import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.function.Function;
 import java.util.stream.Collectors;
@@ -47,6 +47,7 @@ import org.apache.commons.lang3.Validate;
 import org.apache.samza.SamzaException;
 import org.apache.samza.metrics.Counter;
 import org.apache.samza.metrics.MetricsRegistry;
+import org.apache.samza.metrics.SlidingTimeWindowReservoir;
 import org.apache.samza.system.IncomingMessageEnvelope;
 import org.apache.samza.system.SystemStreamPartition;
 import org.apache.samza.system.eventhub.EventHubClientManager;
@@ -57,6 +58,8 @@ import org.apache.samza.system.eventhub.admin.EventHubSystemAdmin;
 import org.apache.samza.system.eventhub.metrics.SamzaHistogram;
 import org.apache.samza.system.eventhub.producer.EventHubSystemProducer;
 import org.apache.samza.util.BlockingEnvelopeMap;
+import org.apache.samza.util.Clock;
+import org.apache.samza.util.ShutdownUtil;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -102,7 +105,7 @@ public class EventHubSystemConsumer extends BlockingEnvelopeMap {
 
   // Overall timeout for EventHubClient exponential backoff policy
   private static final Duration DEFAULT_EVENTHUB_RECEIVER_TIMEOUT = Duration.ofMinutes(10L);
-  private static final long DEFAULT_SHUTDOWN_TIMEOUT_MILLIS = Duration.ofMinutes(1L).toMillis();
+  private static final long DEFAULT_SHUTDOWN_TIMEOUT_MILLIS = Duration.ofSeconds(15).toMillis();
 
   public static final String START_OF_STREAM = ClientConstants.START_OF_STREAM; // -1
   public static final String END_OF_STREAM = "-2";
@@ -140,22 +143,39 @@ public class EventHubSystemConsumer extends BlockingEnvelopeMap {
   private final String systemName;
   private final EventHubClientManagerFactory eventHubClientManagerFactory;
 
-  // Partition receiver error propagation
-  private final AtomicReference<Throwable> eventHubHandlerError = new AtomicReference<>(null);
+  // Partition receiver non transient error propagation
+  private final AtomicReference<Throwable> eventHubNonTransientError = new AtomicReference<>(null);
+
+  private final ExecutorService reconnectTaskRunner = Executors.newSingleThreadExecutor(
+      new ThreadFactoryBuilder().setNameFormat("EventHubs-Reconnect-Task").setDaemon(true).build());
+  private long lastRetryTs = 0;
+
+  private final Clock clock;
+  @VisibleForTesting
+  final SlidingTimeWindowReservoir recentRetryAttempts;
+  @VisibleForTesting
+  volatile Future reconnectTaskStatus = null;
 
   public EventHubSystemConsumer(EventHubConfig config, String systemName,
       EventHubClientManagerFactory eventHubClientManagerFactory, Map<String, Interceptor> interceptors,
       MetricsRegistry registry) {
-    super(registry, System::currentTimeMillis);
+    this(config, systemName, eventHubClientManagerFactory, interceptors, registry, System::currentTimeMillis);
+  }
+
+  EventHubSystemConsumer(EventHubConfig config, String systemName,
+      EventHubClientManagerFactory eventHubClientManagerFactory, Map<String, Interceptor> interceptors,
+      MetricsRegistry registry, Clock clock) {
+    super(registry, clock);
 
     this.config = config;
+    this.clock = clock;
     this.systemName = systemName;
     this.interceptors = interceptors;
     this.eventHubClientManagerFactory = eventHubClientManagerFactory;
     List<String> streamIds = config.getStreams(systemName);
     prefetchCount = config.getPrefetchCount(systemName);
 
-
+    recentRetryAttempts = new SlidingTimeWindowReservoir(config.getRetryWindowMs(systemName), clock);
 
     // Initiate metrics
     eventReadRates =
@@ -234,14 +254,9 @@ public class EventHubSystemConsumer extends BlockingEnvelopeMap {
     return eventHubClientManager;
   }
 
-  @Override
-  public void start() {
-    if (isStarted) {
-      LOG.warn("Trying to start EventHubSystemConsumer while it's already started. Ignore the request.");
-      return;
-    }
-    isStarted = true;
+  private synchronized void initializeEventHubsManagers() {
     LOG.info("Starting EventHubSystemConsumer. Count of SSPs registered: " + streamPartitionOffsets.entrySet().size());
+    eventHubNonTransientError.set(null);
     // Create receivers for Event Hubs
     for (Map.Entry<SystemStreamPartition, String> entry : streamPartitionOffsets.entrySet()) {
       SystemStreamPartition ssp = entry.getKey();
@@ -261,13 +276,11 @@ public class EventHubSystemConsumer extends BlockingEnvelopeMap {
           receiver = eventHubClientManager.getEventHubClient()
               .createReceiverSync(consumerGroup, partitionId.toString(), EventPosition.fromEnqueuedTime(Instant.now()));
         } else {
-          // If the offset is less or equal to the newest offset in the system, it can be
-          // used as the starting offset to receive from. EventHub will return the first
-          // message AFTER the offset that was specified in the fetch request.
+          // EventHub will return the first message AFTER the offset that was specified in the fetch request.
           // If no such offset exists Eventhub will return an error.
           receiver = eventHubClientManager.getEventHubClient()
               .createReceiverSync(consumerGroup, partitionId.toString(),
-                  EventPosition.fromOffset(offset, !offset.equals(EventHubSystemConsumer.START_OF_STREAM)));
+                  EventPosition.fromOffset(offset, /* inclusiveFlag */false));
         }
 
         receiver.setPrefetchCount(prefetchCount);
@@ -292,25 +305,43 @@ public class EventHubSystemConsumer extends BlockingEnvelopeMap {
       }
       LOG.info(String.format("Connection successfully started for namespace=%s, entity=%s ", namespace, entityPath));
     }
+  }
+
+  @Override
+  public void start() {
+    if (isStarted) {
+      LOG.warn("Trying to start EventHubSystemConsumer while it's already started. Ignore the request.");
+      return;
+    }
+    isStarted = true;
+    initializeEventHubsManagers();
     LOG.info("EventHubSystemConsumer started");
   }
 
   @Override
   public Map<SystemStreamPartition, List<IncomingMessageEnvelope>> poll(
       Set<SystemStreamPartition> systemStreamPartitions, long timeout) throws InterruptedException {
-    Throwable handlerError = eventHubHandlerError.get();
-
-    if (handlerError != null) {
-      if (isErrorTransient(handlerError)) {
-        // Log a warning if the error is transient
-        // Partition receiver handler OnError should have handled it by recreating the receiver
-        LOG.warn("Received a transient error from event hub partition receiver, restarted receiver", handlerError);
+    Throwable handlerError = eventHubNonTransientError.get();
+    /*
+     * We will retry for non transient error by instantiating a new EventHubs client if
+     * 1. Last retry happened more than CONFIG_MIN_RETRY_INTERVAL_MS ms ago. Otherwise we ignore
+     * 2. We haven't reached CONFIG_MAX_RETRY_COUNT allowed within the CONFIG_RETRY_WINDOW_MS window.
+     *    Otherwise we throw
+     */
+    if (handlerError != null && clock.currentTimeMillis() - lastRetryTs > config.getMinRetryIntervalMs(systemName)) {
+      int currentRetryCount = recentRetryAttempts.size();
+      long maxRetryCount = config.getMaxRetryCount(systemName);
+      if (currentRetryCount < maxRetryCount) {
+        LOG.warn("Received non transient error. Will retry.", handlerError);
+        LOG.info("Current retry count within window: {}. max retry count allowed: {}. window size: {} ms",
+            currentRetryCount, maxRetryCount, config.getRetryWindowMs(systemName));
+        long now = clock.currentTimeMillis();
+        recentRetryAttempts.update(now);
+        lastRetryTs = now;
+        reconnectTaskStatus = reconnectTaskRunner.submit(this::renewEventHubsClient);
       } else {
-        // Propagate the error to user if the throwable is either
-        // 1. permanent ServiceBusException error from client
-        // 2. SamzaException thrown bu the EventHubConsumer
-        //   2a. Interrupted during put operation to BEM
-        //   2b. Failure in renewing the Partititon Receiver
+        LOG.error("Retries exhausted. Reached max allowed retries: ({}) within window {} ms", currentRetryCount,
+            config.getRetryWindowMs(systemName));
         String msg = "Received a non transient error from event hub partition receiver";
         throw new SamzaException(msg, handlerError);
       }
@@ -319,6 +350,17 @@ public class EventHubSystemConsumer extends BlockingEnvelopeMap {
     return super.poll(systemStreamPartitions, timeout);
   }
 
+  private synchronized void renewEventHubsClient() {
+    try {
+      LOG.info("Start to renew eventhubs client");
+      shutdownEventHubsManagers(); // The shutdown is in parallel and time bounded
+      initializeEventHubsManagers();
+    } catch (Exception e) {
+      LOG.error("Failed to renew eventhubs client", e);
+      eventHubNonTransientError.set(e);
+    }
+  }
+
   private void renewPartitionReceiver(SystemStreamPartition ssp) {
     String streamId = config.getStreamId(ssp.getStream());
     EventHubClientManager eventHubClientManager = perPartitionEventHubManagers.get(ssp);
@@ -344,39 +386,54 @@ public class EventHubSystemConsumer extends BlockingEnvelopeMap {
       receiver.setReceiveHandler(streamPartitionHandlers.get(ssp));
       streamPartitionReceivers.put(ssp, receiver);
     } catch (Exception e) {
-      eventHubHandlerError.set(new SamzaException(
+      eventHubNonTransientError.set(new SamzaException(
           String.format("Failed to recreate receiver for EventHubs after ReceiverHandlerError (ssp=%s)", ssp), e));
     }
   }
 
+  private synchronized void shutdownEventHubsManagers() {
+    // There could be potentially many Receivers and EventHubManagers, so close the managers in parallel
+    LOG.info("Start shutting down eventhubs receivers");
+    ShutdownUtil.boundedShutdown(streamPartitionReceivers.values().stream().map(receiver -> new Runnable() {
+      @Override
+      public void run() {
+        try {
+          receiver.close().get(DEFAULT_SHUTDOWN_TIMEOUT_MILLIS, TimeUnit.MILLISECONDS);
+        } catch (Exception e) {
+          LOG.error("Failed to shutdown receiver.", e);
+        }
+      }
+    }).collect(Collectors.toList()), "EventHubSystemConsumer.Receiver#close", DEFAULT_SHUTDOWN_TIMEOUT_MILLIS);
+
+    LOG.info("Start shutting down eventhubs managers");
+    ShutdownUtil.boundedShutdown(perPartitionEventHubManagers.values().stream().map(manager -> new Runnable() {
+      @Override
+      public void run() {
+        try {
+          manager.close(DEFAULT_SHUTDOWN_TIMEOUT_MILLIS);
+        } catch (Exception e) {
+          LOG.error("Failed to shutdown eventhubs manager.", e);
+        }
+      }
+    }).collect(Collectors.toList()), "EventHubSystemConsumer.ClientManager#close", DEFAULT_SHUTDOWN_TIMEOUT_MILLIS);
+
+    perPartitionEventHubManagers.clear();
+    perStreamEventHubManagers.clear();
+  }
+
   @Override
   public void stop() {
     LOG.info("Stopping event hub system consumer...");
-    List<CompletableFuture<Void>> futures = new ArrayList<>();
-    streamPartitionReceivers.values().forEach((receiver) -> futures.add(receiver.close()));
-    CompletableFuture<Void> future = CompletableFuture.allOf(futures.toArray(new CompletableFuture[futures.size()]));
     try {
-      future.get(DEFAULT_SHUTDOWN_TIMEOUT_MILLIS, TimeUnit.MILLISECONDS);
-    } catch (ExecutionException | InterruptedException | TimeoutException e) {
-      LOG.warn("Failed to close receivers", e);
+      reconnectTaskRunner.shutdown();
+      shutdownEventHubsManagers();
+      isStarted = false;
+    } catch (Exception e) {
+      LOG.warn("Exception during stop.", e);
     }
-    perPartitionEventHubManagers.values()
-        .parallelStream()
-        .forEach(ehClientManager -> ehClientManager.close(DEFAULT_SHUTDOWN_TIMEOUT_MILLIS));
-    perPartitionEventHubManagers.clear();
-    perStreamEventHubManagers.clear();
-    isStarted = false;
     LOG.info("Event hub system consumer stopped.");
   }
 
-  private boolean isErrorTransient(Throwable throwable) {
-    if (throwable instanceof EventHubException) {
-      EventHubException eventHubException = (EventHubException) throwable;
-      return eventHubException.getIsTransient();
-    }
-    return false;
-  }
-
   @Override
   protected BlockingQueue<IncomingMessageEnvelope> newBlockingQueue() {
     return new LinkedBlockingQueue<>(config.getConsumerBufferCapacity(systemName));
@@ -459,13 +516,15 @@ public class EventHubSystemConsumer extends BlockingEnvelopeMap {
         EventHubException busException = (EventHubException) throwable;
 
         if (busException.getIsTransient()) {
-
-          // Only set to transient throwable if there has been no previous errors
-          eventHubHandlerError.compareAndSet(null, throwable);
-
           LOG.warn(
               String.format("Received transient exception from EH client. Renew partition receiver for ssp: %s", ssp),
               throwable);
+          try {
+            // Add a fixed delay so that we don't keep retrying when there are long-lasting failures
+            Thread.sleep(Duration.ofSeconds(2).toMillis());
+          } catch (InterruptedException e) {
+            LOG.warn("Interrupted during sleep before renew", e);
+          }
           // Retry creating a receiver since error likely due to timeout
           renewPartitionReceiver(ssp);
           return;
@@ -474,7 +533,7 @@ public class EventHubSystemConsumer extends BlockingEnvelopeMap {
 
       LOG.error(String.format("Received non transient exception from EH client for ssp: %s", ssp), throwable);
       // Propagate non transient or unknown errors
-      eventHubHandlerError.set(throwable);
+      eventHubNonTransientError.set(throwable);
     }
   }
 }
