@@ -26,7 +26,7 @@ import org.apache.samza.coordinator.JobModelManager;
 import org.apache.samza.coordinator.stream.messages.SetContainerHostMapping;
 import org.apache.samza.metrics.ContainerProcessManagerMetrics;
 import org.apache.samza.metrics.MetricsRegistryMap;
-import org.apache.samza.util.ClassLoaderHelper;
+import org.apache.samza.util.Util;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import java.util.HashMap;
@@ -105,6 +105,18 @@ public class ContainerProcessManager implements ClusterResourceManager.Callback
    */
   private final ContainerProcessManagerMetrics metrics;
 
+  //for testing
+  ContainerProcessManager(Config config, SamzaApplicationState state, MetricsRegistryMap registry, AbstractContainerAllocator allocator, ClusterResourceManager manager) {
+    this.state = state;
+    this.clusterManagerConfig = new ClusterManagerConfig(config);
+    this.jobConfig = new JobConfig(config);
+    this.hostAffinityEnabled = clusterManagerConfig.getHostAffinityEnabled();
+    this.clusterResourceManager = manager;
+    this.metrics = new ContainerProcessManagerMetrics(config, state, registry);
+    this.containerAllocator = allocator;
+    this.allocatorThread = new Thread(this.containerAllocator, "Container Allocator Thread");
+  }
+
   public ContainerProcessManager(Config config,
                                  SamzaApplicationState state,
                                  MetricsRegistryMap registry) {
@@ -153,7 +165,6 @@ public class ContainerProcessManager implements ClusterResourceManager.Callback
 
     this.allocatorThread = new Thread(this.containerAllocator, "Container Allocator Thread");
     log.info("finished initialization of samza task manager");
-
   }
 
   public boolean shouldShutdown() {
@@ -383,6 +394,50 @@ public class ContainerProcessManager implements ClusterResourceManager.Callback
     }
   }
 
+  @Override
+  public void onStreamProcessorLaunchSuccess(SamzaResource resource) {
+
+    // 1. Obtain the Samza container Id for the pending container on this resource.
+    String containerId = getPendingContainerId(resource.getResourceID());
+    log.info("Successfully started container ID: {} on resource: {}", containerId, resource);
+
+    // 2. Remove the container from the pending buffer and add it to the running buffer. Additionally, update the
+    // job-health metric.
+    if (containerId != null) {
+      log.info("Moving containerID: {} on resource: {} from pending to running state", containerId, resource);
+      state.pendingContainers.remove(containerId);
+      state.runningContainers.put(containerId, resource);
+
+      if (state.neededContainers.decrementAndGet() == 0) {
+        state.jobHealthy.set(true);
+      }
+    } else {
+      log.warn("SamzaResource {} was not in pending state. Got an invalid callback for a launch request that " +
+          "was not issued", resource);
+    }
+  }
+
+  @Override
+  public void onStreamProcessorLaunchFailure(SamzaResource resource, Throwable t) {
+    log.error("Got a launch failure for SamzaResource {} with exception {}", resource, t);
+    // 1. Release resources for containers that failed back to YARN
+    log.info("Releasing unstartable container {}", resource.getResourceID());
+    clusterResourceManager.releaseResources(resource);
+
+    // 2. Obtain the Samza container Id for the pending container on this resource.
+    String containerId = getPendingContainerId(resource.getResourceID());
+    log.info("Failed container ID: {} for resourceId: {}", containerId, resource.getResourceID());
+
+    // 3. Re-request resources on ANY_HOST in case of launch failures on the preferred host.
+    if (containerId != null) {
+      log.info("Launch of container ID: {} failed on host: {}. Falling back to ANY_HOST", containerId, resource.getHost());
+      containerAllocator.requestResource(containerId, ResourceRequestState.ANY_HOST);
+    } else {
+      log.warn("SamzaResource {} was not in pending state. Got an invalid callback for a launch request that was " +
+          "not issued", resource);
+    }
+  }
+
   /**
    * An error in the callback terminates the JobCoordinator
    * @param e the underlying exception/error
@@ -405,20 +460,29 @@ public class ContainerProcessManager implements ClusterResourceManager.Callback
     final ResourceManagerFactory factory;
 
     try {
-      factory = ClassLoaderHelper.<ResourceManagerFactory>fromClassName(containerManagerFactoryClass);
-    } catch (InstantiationException e) {
-      log.error("Instantiation exception when creating ContainerManager", e);
-      throw new SamzaException(e);
-    } catch (IllegalAccessException e) {
-      log.error("Illegal access exception when creating ContainerManager", e);
-      throw new SamzaException(e);
-    } catch (ClassNotFoundException e) {
-      log.error("ClassNotFound Exception when creating ContainerManager", e);
+      factory = Util.getObj(containerManagerFactoryClass, ResourceManagerFactory.class);
+    } catch (Exception e) {
+      log.error("Exception when creating ContainerManager", e);
       throw new SamzaException(e);
     }
     return factory;
   }
 
+  /**
+   * Obtains the ID of the Samza container pending launch on the provided resource.
+   *
+   * @param resourceId the Id of the resource
+   * @return the Id of the Samza container on this resource
+   */
+  private String getPendingContainerId(String resourceId) {
+    for (Map.Entry<String, SamzaResource> entry: state.pendingContainers.entrySet()) {
+      if (entry.getValue().getResourceID().equals(resourceId)) {
+        log.info("Matching container ID found " + entry.getKey() + " " + entry.getValue());
+        return entry.getKey();
+      }
+    }
+    return null;
+  }
 
 
-}
+}
\ No newline at end of file
