@@ -79,6 +79,7 @@ public class HostAwareContainerAllocator extends AbstractContainerAllocator {
         boolean resourceAvailableOnAnyHost = hasAllocatedResource(ResourceRequestState.ANY_HOST);
 
         if (expired) {
+          updateExpiryMetrics(request);
           if (resourceAvailableOnAnyHost) {
             log.info("Request for container: {} on {} has expired. Running on ANY_HOST", request.getContainerID(), request.getPreferredHost());
             runStreamProcessor(request, ResourceRequestState.ANY_HOST);
@@ -109,4 +110,13 @@ public class HostAwareContainerAllocator extends AbstractContainerAllocator {
     }
     return requestExpired;
   }
+
+  private void updateExpiryMetrics(SamzaResourceRequest request) {
+    String preferredHost = request.getPreferredHost();
+    if (ResourceRequestState.ANY_HOST.equals(preferredHost)) {
+      state.expiredAnyHostRequests.incrementAndGet();
+    } else {
+      state.expiredPreferredHostRequests.incrementAndGet();
+    }
+  }
 }
