@@ -83,6 +83,16 @@ public class JavaSystemConfig extends MapConfig {
                 .getAdmin(systemNameToFactoryEntry.getKey(), this)));
   }
 
+  /**
+   * Get {@link SystemAdmin} instance for given system name.
+   *
+   * @param systemName System name
+   * @return SystemAdmin of the system if it exists, otherwise null.
+   */
+  public SystemAdmin getSystemAdmin(String systemName) {
+    return getSystemAdmins().get(systemName);
+  }
+
   /**
    * Get {@link SystemFactory} instances for all the systems defined in this config.
    *
@@ -97,7 +107,7 @@ public class JavaSystemConfig extends MapConfig {
           throw new SamzaException(
               String.format("A stream uses system %s, which is missing from the configuration.", systemName));
         }
-        return Util.getObj(systemFactoryClassName);
+        return Util.getObj(systemFactoryClassName, SystemFactory.class);
       }));
 
     return systemFactories;
