@@ -19,8 +19,6 @@
 package org.apache.samza.container.grouper.task;
 
 import org.apache.samza.config.Config;
-import org.apache.samza.config.JobConfig;
-
 
 /**
  * Factory to build the GroupByContainerCount class.
@@ -28,6 +26,6 @@ import org.apache.samza.config.JobConfig;
 public class GroupByContainerCountFactory implements TaskNameGrouperFactory {
   @Override
   public TaskNameGrouper build(Config config) {
-    return new GroupByContainerCount(new JobConfig(config).getContainerCount());
+    return new GroupByContainerCount(config);
   }
 }
