@@ -22,14 +22,15 @@ import com.google.common.collect.HashMultimap;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Multimap;
 import org.apache.samza.config.Config;
-import org.apache.samza.container.TaskContextImpl;
+import org.apache.samza.config.StreamConfig;
+import org.apache.samza.context.Context;
+import org.apache.samza.context.TaskContextImpl;
 import org.apache.samza.job.model.JobModel;
 import org.apache.samza.operators.KV;
-import org.apache.samza.operators.StreamGraphImpl;
-import org.apache.samza.operators.TimerRegistry;
+import org.apache.samza.operators.OperatorSpecGraph;
+import org.apache.samza.operators.Scheduler;
 import org.apache.samza.operators.functions.JoinFunction;
 import org.apache.samza.operators.functions.PartialJoinFunction;
-import org.apache.samza.operators.impl.store.TimestampedValue;
 import org.apache.samza.operators.spec.BroadcastOperatorSpec;
 import org.apache.samza.operators.spec.InputOperatorSpec;
 import org.apache.samza.operators.spec.JoinOperatorSpec;
@@ -43,8 +44,8 @@ import org.apache.samza.operators.spec.StreamTableJoinOperatorSpec;
 import org.apache.samza.operators.spec.WindowOperatorSpec;
 import org.apache.samza.storage.kv.KeyValueStore;
 import org.apache.samza.system.SystemStream;
-import org.apache.samza.task.TaskContext;
 import org.apache.samza.util.Clock;
+import org.apache.samza.util.TimestampedValue;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -81,42 +82,45 @@ public class OperatorImplGraph {
    * the two {@link PartialJoinOperatorImpl}s for a {@link JoinOperatorSpec} with each other since they're
    * reached from different {@link OperatorSpec} during DAG traversals.
    */
-  private final Map<String, KV<PartialJoinFunction, PartialJoinFunction>> joinFunctions = new HashMap<>();
+  private final Map<String, KV<PartialJoinOperatorImpl, PartialJoinOperatorImpl>> joinOpImpls = new HashMap<>();
 
   private final Clock clock;
 
   /**
    * Constructs the DAG of {@link OperatorImpl}s corresponding to the the DAG of {@link OperatorSpec}s
-   * in the {@code streamGraph}.
+   * in the {@code specGraph}.
    *
-   * @param streamGraph  the {@link StreamGraphImpl} containing the logical {@link OperatorSpec} DAG
-   * @param config  the {@link Config} required to instantiate operators
-   * @param context  the {@link TaskContext} required to instantiate operators
+   * @param specGraph  the {@link OperatorSpecGraph} containing the logical {@link OperatorSpec} DAG
+   * @param context  the {@link Context} required to instantiate operators
    * @param clock  the {@link Clock} to get current time
    */
-  public OperatorImplGraph(StreamGraphImpl streamGraph, Config config, TaskContext context, Clock clock) {
+  public OperatorImplGraph(OperatorSpecGraph specGraph, Context context, Clock clock) {
     this.clock = clock;
-
-    TaskContextImpl taskContext = (TaskContextImpl) context;
-    Map<SystemStream, Integer> producerTaskCounts = hasIntermediateStreams(streamGraph) ?
-        getProducerTaskCountForIntermediateStreams(getStreamToConsumerTasks(taskContext.getJobModel()),
-            getIntermediateToInputStreamsMap(streamGraph)) :
-        Collections.EMPTY_MAP;
+    StreamConfig streamConfig = new StreamConfig(context.getJobContext().getConfig());
+    // TODO SAMZA-1935: the objects that are only accessible through TaskContextImpl should be moved somewhere else
+    TaskContextImpl taskContext = (TaskContextImpl) context.getTaskContext();
+    Map<SystemStream, Integer> producerTaskCounts =
+        hasIntermediateStreams(specGraph)
+            ? getProducerTaskCountForIntermediateStreams(
+                getStreamToConsumerTasks(taskContext.getJobModel()),
+                getIntermediateToInputStreamsMap(specGraph, streamConfig))
+            : Collections.EMPTY_MAP;
     producerTaskCounts.forEach((stream, count) -> {
         LOG.info("{} has {} producer tasks.", stream, count);
       });
 
     // set states for end-of-stream
     taskContext.registerObject(EndOfStreamStates.class.getName(),
-        new EndOfStreamStates(context.getSystemStreamPartitions(), producerTaskCounts));
+        new EndOfStreamStates(taskContext.getTaskModel().getSystemStreamPartitions(), producerTaskCounts));
     // set states for watermark
     taskContext.registerObject(WatermarkStates.class.getName(),
-        new WatermarkStates(context.getSystemStreamPartitions(), producerTaskCounts));
+        new WatermarkStates(taskContext.getTaskModel().getSystemStreamPartitions(), producerTaskCounts,
+            context.getContainerContext().getContainerMetricsRegistry()));
 
-    streamGraph.getInputOperators().forEach((streamSpec, inputOpSpec) -> {
-        SystemStream systemStream = new SystemStream(streamSpec.getSystemName(), streamSpec.getPhysicalName());
+    specGraph.getInputOperators().forEach((streamId, inputOpSpec) -> {
+        SystemStream systemStream = streamConfig.streamIdToSystemStream(streamId);
         InputOperatorImpl inputOperatorImpl =
-            (InputOperatorImpl) createAndRegisterOperatorImpl(null, inputOpSpec, systemStream, config, context);
+            (InputOperatorImpl) createAndRegisterOperatorImpl(null, inputOpSpec, systemStream, context);
         this.inputOperators.put(systemStream, inputOperatorImpl);
       });
   }
@@ -151,24 +155,23 @@ public class OperatorImplGraph {
    * creates the corresponding DAG of {@link OperatorImpl}s, and returns the root {@link OperatorImpl} node.
    *
    * @param prevOperatorSpec  the parent of the current {@code operatorSpec} in the traversal
-   * @param operatorSpec  the operatorSpec to create the {@link OperatorImpl} for
-   * @param config  the {@link Config} required to instantiate operators
-   * @param context  the {@link TaskContext} required to instantiate operators
+   * @param operatorSpec  the {@link OperatorSpec} to create the {@link OperatorImpl} for
+   * @param inputStream  the source input stream that we traverse the {@link OperatorSpecGraph} from
+   * @param context the {@link Context} required to instantiate operators
    * @return  the operator implementation for the operatorSpec
    */
-  OperatorImpl createAndRegisterOperatorImpl(OperatorSpec prevOperatorSpec, OperatorSpec operatorSpec,
-      SystemStream inputStream, Config config, TaskContext context) {
-
+  private OperatorImpl createAndRegisterOperatorImpl(OperatorSpec prevOperatorSpec, OperatorSpec operatorSpec,
+      SystemStream inputStream, Context context) {
     if (!operatorImpls.containsKey(operatorSpec.getOpId()) || operatorSpec instanceof JoinOperatorSpec) {
       // Either this is the first time we've seen this operatorSpec, or this is a join operator spec
       // and we need to create 2 partial join operator impls for it. Initialize and register the sub-DAG.
-      OperatorImpl operatorImpl = createOperatorImpl(prevOperatorSpec, operatorSpec, config, context);
-      operatorImpl.init(config, context);
+      OperatorImpl operatorImpl = createOperatorImpl(prevOperatorSpec, operatorSpec, context);
+      operatorImpl.init(context);
       operatorImpl.registerInputStream(inputStream);
 
-      if (operatorSpec.getTimerFn() != null) {
-        final TimerRegistry timerRegistry = operatorImpl.createOperatorTimerRegistry();
-        operatorSpec.getTimerFn().registerTimer(timerRegistry);
+      if (operatorSpec.getScheduledFn() != null) {
+        final Scheduler scheduler = operatorImpl.createOperatorScheduler();
+        operatorSpec.getScheduledFn().schedule(scheduler);
       }
 
       // Note: The key here is opImplId, which may not equal opId for some impls (e.g. PartialJoinOperatorImpl).
@@ -178,7 +181,8 @@ public class OperatorImplGraph {
 
       Collection<OperatorSpec> registeredSpecs = operatorSpec.getRegisteredOperatorSpecs();
       registeredSpecs.forEach(registeredSpec -> {
-          OperatorImpl nextImpl = createAndRegisterOperatorImpl(operatorSpec, registeredSpec, inputStream, config, context);
+          LOG.debug("Creating operator {} with opCode: {}", registeredSpec.getOpId(), registeredSpec.getOpCode());
+          OperatorImpl nextImpl = createAndRegisterOperatorImpl(operatorSpec, registeredSpec, inputStream, context);
           operatorImpl.registerNextOperator(nextImpl);
         });
       return operatorImpl;
@@ -189,9 +193,8 @@ public class OperatorImplGraph {
 
       // We still need to traverse the DAG further to register the input streams.
       Collection<OperatorSpec> registeredSpecs = operatorSpec.getRegisteredOperatorSpecs();
-      registeredSpecs.forEach(registeredSpec -> {
-          createAndRegisterOperatorImpl(operatorSpec, registeredSpec, inputStream, config, context);
-        });
+      registeredSpecs.forEach(
+          registeredSpec -> createAndRegisterOperatorImpl(operatorSpec, registeredSpec, inputStream, context));
       return operatorImpl;
     }
   }
@@ -199,53 +202,62 @@ public class OperatorImplGraph {
   /**
    * Creates a new {@link OperatorImpl} instance for the provided {@link OperatorSpec}.
    *
-   * @param operatorSpec  the immutable {@link OperatorSpec} definition.
-   * @param config  the {@link Config} required to instantiate operators
-   * @param context  the {@link TaskContext} required to instantiate operators
+   * @param prevOperatorSpec the original {@link OperatorSpec} that produces output for {@code operatorSpec} from {@link OperatorSpecGraph}
+   * @param operatorSpec  the original {@link OperatorSpec} from {@link OperatorSpecGraph}
+   * @param context  the {@link Context} required to instantiate operators
    * @return  the {@link OperatorImpl} implementation instance
    */
-  OperatorImpl createOperatorImpl(OperatorSpec prevOperatorSpec, OperatorSpec operatorSpec,
-      Config config, TaskContext context) {
+  OperatorImpl createOperatorImpl(OperatorSpec prevOperatorSpec, OperatorSpec operatorSpec, Context context) {
+    Config config = context.getJobContext().getConfig();
+    StreamConfig streamConfig = new StreamConfig(config);
     if (operatorSpec instanceof InputOperatorSpec) {
       return new InputOperatorImpl((InputOperatorSpec) operatorSpec);
     } else if (operatorSpec instanceof StreamOperatorSpec) {
-      return new StreamOperatorImpl((StreamOperatorSpec) operatorSpec, config, context);
+      return new StreamOperatorImpl((StreamOperatorSpec) operatorSpec);
     } else if (operatorSpec instanceof SinkOperatorSpec) {
-      return new SinkOperatorImpl((SinkOperatorSpec) operatorSpec, config, context);
+      return new SinkOperatorImpl((SinkOperatorSpec) operatorSpec);
     } else if (operatorSpec instanceof OutputOperatorSpec) {
-      return new OutputOperatorImpl((OutputOperatorSpec) operatorSpec, config, context);
+      String streamId = ((OutputOperatorSpec) operatorSpec).getOutputStream().getStreamId();
+      SystemStream systemStream = streamConfig.streamIdToSystemStream(streamId);
+      return new OutputOperatorImpl((OutputOperatorSpec) operatorSpec, systemStream);
     } else if (operatorSpec instanceof PartitionByOperatorSpec) {
-      return new PartitionByOperatorImpl((PartitionByOperatorSpec) operatorSpec, config, context);
+      String streamId = ((PartitionByOperatorSpec) operatorSpec).getOutputStream().getStreamId();
+      SystemStream systemStream = streamConfig.streamIdToSystemStream(streamId);
+      return new PartitionByOperatorImpl((PartitionByOperatorSpec) operatorSpec, systemStream, context);
     } else if (operatorSpec instanceof WindowOperatorSpec) {
       return new WindowOperatorImpl((WindowOperatorSpec) operatorSpec, clock);
     } else if (operatorSpec instanceof JoinOperatorSpec) {
-      return createPartialJoinOperatorImpl(prevOperatorSpec, (JoinOperatorSpec) operatorSpec, config, context, clock);
+      return getOrCreatePartialJoinOpImpls((JoinOperatorSpec) operatorSpec,
+          prevOperatorSpec.equals(((JoinOperatorSpec) operatorSpec).getLeftInputOpSpec()), clock);
     } else if (operatorSpec instanceof StreamTableJoinOperatorSpec) {
-      return new StreamTableJoinOperatorImpl((StreamTableJoinOperatorSpec) operatorSpec, config, context);
+      return new StreamTableJoinOperatorImpl((StreamTableJoinOperatorSpec) operatorSpec, context);
     } else if (operatorSpec instanceof SendToTableOperatorSpec) {
-      return new SendToTableOperatorImpl((SendToTableOperatorSpec) operatorSpec, config, context);
+      return new SendToTableOperatorImpl((SendToTableOperatorSpec) operatorSpec, context);
     } else if (operatorSpec instanceof BroadcastOperatorSpec) {
-      return new BroadcastOperatorImpl((BroadcastOperatorSpec) operatorSpec, context);
+      String streamId = ((BroadcastOperatorSpec) operatorSpec).getOutputStream().getStreamId();
+      SystemStream systemStream = streamConfig.streamIdToSystemStream(streamId);
+      return new BroadcastOperatorImpl((BroadcastOperatorSpec) operatorSpec, systemStream, context);
     }
     throw new IllegalArgumentException(
         String.format("Unsupported OperatorSpec: %s", operatorSpec.getClass().getName()));
   }
 
-  private PartialJoinOperatorImpl createPartialJoinOperatorImpl(OperatorSpec prevOperatorSpec,
-      JoinOperatorSpec joinOpSpec, Config config, TaskContext context, Clock clock) {
-    KV<PartialJoinFunction, PartialJoinFunction> partialJoinFunctions = getOrCreatePartialJoinFunctions(joinOpSpec);
-    if (joinOpSpec.getLeftInputOpSpec().equals(prevOperatorSpec)) { // we got here from the left side of the join
-      return new PartialJoinOperatorImpl(joinOpSpec, /* isLeftSide */ true,
-          partialJoinFunctions.getKey(), partialJoinFunctions.getValue(), config, context, clock);
-    } else { // we got here from the right side of the join
-      return new PartialJoinOperatorImpl(joinOpSpec, /* isLeftSide */ false,
-          partialJoinFunctions.getValue(), partialJoinFunctions.getKey(), config, context, clock);
-    }
-  }
+  private PartialJoinOperatorImpl getOrCreatePartialJoinOpImpls(JoinOperatorSpec joinOpSpec, boolean isLeft,
+      Clock clock) {
+    // get the per task pair of PartialJoinOperatorImpl for the corresponding {@code joinOpSpec}
+    KV<PartialJoinOperatorImpl, PartialJoinOperatorImpl> partialJoinOpImpls = joinOpImpls.computeIfAbsent(joinOpSpec.getOpId(),
+        joinOpId -> {
+        PartialJoinFunction leftJoinFn = createLeftJoinFn(joinOpSpec);
+        PartialJoinFunction rightJoinFn = createRightJoinFn(joinOpSpec);
+        return new KV(new PartialJoinOperatorImpl(joinOpSpec, true, leftJoinFn, rightJoinFn, clock),
+            new PartialJoinOperatorImpl(joinOpSpec, false, rightJoinFn, leftJoinFn, clock));
+      });
 
-  private KV<PartialJoinFunction, PartialJoinFunction> getOrCreatePartialJoinFunctions(JoinOperatorSpec joinOpSpec) {
-    return joinFunctions.computeIfAbsent(joinOpSpec.getOpId(),
-        joinOpId -> KV.of(createLeftJoinFn(joinOpSpec), createRightJoinFn(joinOpSpec)));
+    if (isLeft) { // we got here from the left side of the join
+      return partialJoinOpImpls.getKey();
+    } else { // we got here from the right side of the join
+      return partialJoinOpImpls.getValue();
+    }
   }
 
   private PartialJoinFunction<Object, Object, Object, Object> createLeftJoinFn(JoinOperatorSpec joinOpSpec) {
@@ -269,12 +281,13 @@ public class OperatorImplGraph {
       }
 
       @Override
-      public void init(Config config, TaskContext context) {
+      public void init(Context context) {
         String leftStoreName = joinOpSpec.getLeftOpId();
-        leftStreamState = (KeyValueStore<Object, TimestampedValue<Object>>) context.getStore(leftStoreName);
+        leftStreamState =
+            (KeyValueStore<Object, TimestampedValue<Object>>) context.getTaskContext().getStore(leftStoreName);
 
         // user-defined joinFn should only be initialized once, so we do it only in left partial join function.
-        joinFn.init(config, context);
+        joinFn.init(context);
       }
 
       @Override
@@ -301,9 +314,10 @@ public class OperatorImplGraph {
       }
 
       @Override
-      public void init(Config config, TaskContext context) {
+      public void init(Context context) {
         String rightStoreName = joinOpSpec.getRightOpId();
-        rightStreamState = (KeyValueStore<Object, TimestampedValue<Object>>) context.getStore(rightStoreName);
+        rightStreamState =
+            (KeyValueStore<Object, TimestampedValue<Object>>) context.getTaskContext().getStore(rightStoreName);
 
         // user-defined joinFn should only be initialized once,
         // so we do it only in left partial join function and not here again.
@@ -316,10 +330,6 @@ public class OperatorImplGraph {
     };
   }
 
-  private boolean hasIntermediateStreams(StreamGraphImpl streamGraph) {
-    return !Collections.disjoint(streamGraph.getInputOperators().keySet(), streamGraph.getOutputStreams().keySet());
-  }
-
   /**
    * calculate the task count that produces to each intermediate streams
    * @param streamToConsumerTasks input streams to task mapping
@@ -330,12 +340,11 @@ public class OperatorImplGraph {
       Multimap<SystemStream, String> streamToConsumerTasks,
       Multimap<SystemStream, SystemStream> intermediateToInputStreams) {
     Map<SystemStream, Integer> result = new HashMap<>();
-    intermediateToInputStreams.asMap().entrySet().forEach(entry -> {
+    intermediateToInputStreams.asMap().entrySet().forEach(entry ->
         result.put(entry.getKey(),
             entry.getValue().stream()
                 .flatMap(systemStream -> streamToConsumerTasks.get(systemStream).stream())
-                .collect(Collectors.toSet()).size());
-      });
+                .collect(Collectors.toSet()).size()));
     return result;
   }
 
@@ -358,28 +367,37 @@ public class OperatorImplGraph {
 
   /**
    * calculate the mapping from output streams to input streams
-   * @param streamGraph the user {@link StreamGraphImpl} instance
+   * @param specGraph the user {@link OperatorSpecGraph}
    * @return mapping from output streams to input streams
    */
-  static Multimap<SystemStream, SystemStream> getIntermediateToInputStreamsMap(StreamGraphImpl streamGraph) {
+  static Multimap<SystemStream, SystemStream> getIntermediateToInputStreamsMap(
+      OperatorSpecGraph specGraph, StreamConfig streamConfig) {
     Multimap<SystemStream, SystemStream> outputToInputStreams = HashMultimap.create();
-    streamGraph.getInputOperators().entrySet().stream()
-        .forEach(
-            entry -> computeOutputToInput(entry.getKey().toSystemStream(), entry.getValue(), outputToInputStreams));
+    specGraph.getInputOperators().entrySet().stream()
+        .forEach(entry -> {
+            SystemStream systemStream = streamConfig.streamIdToSystemStream(entry.getKey());
+            computeOutputToInput(systemStream, entry.getValue(), outputToInputStreams, streamConfig);
+          });
     return outputToInputStreams;
   }
 
   private static void computeOutputToInput(SystemStream input, OperatorSpec opSpec,
-      Multimap<SystemStream, SystemStream> outputToInputStreams) {
+      Multimap<SystemStream, SystemStream> outputToInputStreams, StreamConfig streamConfig) {
     if (opSpec instanceof PartitionByOperatorSpec) {
       PartitionByOperatorSpec spec = (PartitionByOperatorSpec) opSpec;
-      outputToInputStreams.put(spec.getOutputStream().getStreamSpec().toSystemStream(), input);
+      SystemStream systemStream = streamConfig.streamIdToSystemStream(spec.getOutputStream().getStreamId());
+      outputToInputStreams.put(systemStream, input);
     } else if (opSpec instanceof BroadcastOperatorSpec) {
       BroadcastOperatorSpec spec = (BroadcastOperatorSpec) opSpec;
-      outputToInputStreams.put(spec.getOutputStream().getStreamSpec().toSystemStream(), input);
+      SystemStream systemStream = streamConfig.streamIdToSystemStream(spec.getOutputStream().getStreamId());
+      outputToInputStreams.put(systemStream, input);
     } else {
       Collection<OperatorSpec> nextOperators = opSpec.getRegisteredOperatorSpecs();
-      nextOperators.forEach(spec -> computeOutputToInput(input, spec, outputToInputStreams));
+      nextOperators.forEach(spec -> computeOutputToInput(input, spec, outputToInputStreams, streamConfig));
     }
   }
+
+  private boolean hasIntermediateStreams(OperatorSpecGraph specGraph) {
+    return !Collections.disjoint(specGraph.getInputOperators().keySet(), specGraph.getOutputStreams().keySet());
+  }
 }
