@@ -19,6 +19,7 @@
 package org.apache.samza.zk;
 
 import com.google.common.annotations.VisibleForTesting;
+
 import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.HashMap;
@@ -26,6 +27,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Objects;
 import java.util.Set;
+import java.util.concurrent.atomic.AtomicBoolean;
 import org.I0Itec.zkclient.IZkStateListener;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.samza.checkpoint.CheckpointManager;
@@ -41,7 +43,6 @@ import org.apache.samza.container.TaskName;
 import org.apache.samza.coordinator.JobCoordinator;
 import org.apache.samza.coordinator.JobCoordinatorListener;
 import org.apache.samza.coordinator.JobModelManager;
-import org.apache.samza.coordinator.LeaderElector;
 import org.apache.samza.coordinator.LeaderElectorListener;
 import org.apache.samza.job.model.ContainerModel;
 import org.apache.samza.job.model.JobModel;
@@ -59,12 +60,10 @@ import org.apache.zookeeper.Watcher;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import static org.apache.zookeeper.Watcher.Event.KeeperState.*;
-
 /**
  * JobCoordinator for stand alone processor managed via Zookeeper.
  */
-public class ZkJobCoordinator implements JobCoordinator, ZkControllerListener {
+public class ZkJobCoordinator implements JobCoordinator {
   private static final Logger LOG = LoggerFactory.getLogger(ZkJobCoordinator.class);
   // TODO: MetadataCache timeout has to be 0 for the leader so that it can always have the latest information associated
   // with locality. Since host-affinity is not yet implemented, this can be fixed as part of SAMZA-1197
@@ -86,23 +85,25 @@ public class ZkJobCoordinator implements JobCoordinator, ZkControllerListener {
 
   private final ZkUtils zkUtils;
   private final String processorId;
-  private final ZkController zkController;
 
   private final Config config;
   private final ZkBarrierForVersionUpgrade barrier;
   private final ZkJobCoordinatorMetrics metrics;
   private final Map<String, MetricsReporter> reporters;
+  private final ZkLeaderElector leaderElector;
+  private final AtomicBoolean initiatedShutdown = new AtomicBoolean(false);
+  private final StreamMetadataCache streamMetadataCache;
+  private final SystemAdmins systemAdmins;
+  private final int debounceTimeMs;
+  private final Map<TaskName, Integer> changeLogPartitionMap = new HashMap<>();
 
-  private StreamMetadataCache streamMetadataCache = null;
-  private SystemAdmins systemAdmins = null;
-  private ScheduleAfterDebounceTime debounceTimer = null;
   private JobCoordinatorListener coordinatorListener = null;
   private JobModel newJobModel;
-  private int debounceTimeMs;
   private boolean hasCreatedStreams = false;
-  private boolean initiatedShutdown = false;
   private String cachedJobModelVersion = null;
-  private Map<TaskName, Integer> changeLogPartitionMap = new HashMap<>();
+
+  @VisibleForTesting
+  ScheduleAfterDebounceTime debounceTimer;
 
   ZkJobCoordinator(Config config, MetricsRegistry metricsRegistry, ZkUtils zkUtils) {
     this.config = config;
@@ -114,13 +115,8 @@ public class ZkJobCoordinator implements JobCoordinator, ZkControllerListener {
     // setup a listener for a session state change
     // we are mostly interested in "session closed" and "new session created" events
     zkUtils.getZkClient().subscribeStateChanges(new ZkSessionStateChangedListener());
-    LeaderElector leaderElector = new ZkLeaderElector(processorId, zkUtils);
+    leaderElector = new ZkLeaderElector(processorId, zkUtils);
     leaderElector.setLeaderElectorListener(new LeaderElectorListenerImpl());
-    this.zkController = new ZkControllerImpl(processorId, zkUtils, this, leaderElector);
-    this.barrier =  new ZkBarrierForVersionUpgrade(
-        zkUtils.getKeyBuilder().getJobModelVersionBarrierPrefix(),
-        zkUtils,
-        new ZkBarrierListenerImpl());
     this.debounceTimeMs = new JobConfig(config).getDebounceTimeMs();
     this.reporters = MetricsReporterLoader.getMetricsReporters(new MetricsConfig(config), processorId);
     debounceTimer = new ScheduleAfterDebounceTime(processorId);
@@ -128,62 +124,76 @@ public class ZkJobCoordinator implements JobCoordinator, ZkControllerListener {
         LOG.error("Received exception in debounce timer! Stopping the job coordinator", throwable);
         stop();
       });
+    this.barrier =  new ZkBarrierForVersionUpgrade(zkUtils.getKeyBuilder().getJobModelVersionBarrierPrefix(), zkUtils, new ZkBarrierListenerImpl(), debounceTimer);
     systemAdmins = new SystemAdmins(config);
     streamMetadataCache = new StreamMetadataCache(systemAdmins, METADATA_CACHE_TTL_MS, SystemClock.instance());
   }
 
   @Override
   public void start() {
+    ZkKeyBuilder keyBuilder = zkUtils.getKeyBuilder();
+    zkUtils.validateZkVersion();
+    zkUtils.validatePaths(new String[]{keyBuilder.getProcessorsPath(), keyBuilder.getJobModelVersionPath(), keyBuilder
+        .getJobModelPathPrefix()});
+
     startMetrics();
     systemAdmins.start();
-    zkController.register();
+    leaderElector.tryBecomeLeader();
+    zkUtils.subscribeToJobModelVersionChange(new ZkJobModelVersionChangeHandler(zkUtils));
   }
 
   @Override
-  public synchronized void stop() {
+  public void stop() {
     // Make the shutdown idempotent
-    if (initiatedShutdown) {
-      LOG.debug("Job Coordinator shutdown is already in progress!");
-      return;
-    }
+    if (initiatedShutdown.compareAndSet(false, true)) {
 
-    LOG.info("Shutting down Job Coordinator...");
-    initiatedShutdown = true;
-    boolean shutdownSuccessful = false;
+      LOG.info("Shutting down JobCoordinator.");
+      boolean shutdownSuccessful = false;
 
-    // Notify the metrics about abandoning the leadership. Moving it up the chain in the shutdown sequence so that
-    // in case of unclean shutdown, we get notified about lack of leader and we can set up some alerts around the absence of leader.
-    metrics.isLeader.set(false);
+      // Notify the metrics about abandoning the leadership. Moving it up the chain in the shutdown sequence so that
+      // in case of unclean shutdown, we get notified about lack of leader and we can set up some alerts around the absence of leader.
+      metrics.isLeader.set(false);
 
-    try {
-      // todo: what does it mean for coordinator listener to be null? why not have it part of constructor?
-      if (coordinatorListener != null) {
-        coordinatorListener.onJobModelExpired();
+      try {
+        // todo: what does it mean for coordinator listener to be null? why not have it part of constructor?
+        if (coordinatorListener != null) {
+          coordinatorListener.onJobModelExpired();
+        }
+
+        debounceTimer.stopScheduler();
+
+        if (leaderElector.amILeader()) {
+          LOG.info("Resigning leadership for processorId: " + processorId);
+          leaderElector.resignLeadership();
+        }
+
+        LOG.info("Shutting down ZkUtils.");
+        // close zk connection
+        if (zkUtils != null) {
+          zkUtils.close();
+        }
+
+        LOG.debug("Shutting down system admins.");
+        systemAdmins.stop();
+
+        LOG.debug("Shutting down metrics.");
+        shutdownMetrics();
+
+        if (coordinatorListener != null) {
+          coordinatorListener.onCoordinatorStop();
+        }
+
+        shutdownSuccessful = true;
+      } catch (Throwable t) {
+        LOG.error("Encountered errors during job coordinator stop.", t);
+        if (coordinatorListener != null) {
+          coordinatorListener.onCoordinatorFailure(t);
+        }
+      } finally {
+        LOG.info("Job Coordinator shutdown finished with ShutdownComplete=" + shutdownSuccessful);
       }
-
-      debounceTimer.stopScheduler();
-
-      LOG.debug("Shutting down ZkController.");
-      zkController.stop();
-
-      LOG.debug("Shutting down system admins.");
-      systemAdmins.stop();
-
-      LOG.debug("Shutting down metrics.");
-      shutdownMetrics();
-
-      if (coordinatorListener != null) {
-        coordinatorListener.onCoordinatorStop();
-      }
-
-      shutdownSuccessful = true;
-    } catch (Throwable t) {
-      LOG.error("Encountered errors during job coordinator stop.", t);
-      if (coordinatorListener != null) {
-        coordinatorListener.onCoordinatorFailure(t);
-      }
-    } finally {
-      LOG.info("Job Coordinator shutdown finished with ShutdownComplete=" + shutdownSuccessful);
+    } else {
+      LOG.info("Job Coordinator shutdown is in progress!");
     }
   }
 
@@ -215,19 +225,23 @@ public class ZkJobCoordinator implements JobCoordinator, ZkControllerListener {
     return processorId;
   }
 
-  //////////////////////////////////////////////// LEADER stuff ///////////////////////////
-  @Override
+  /*
+   * The leader handles notifications for two types of events:
+   *   1. Changes to the current set of processors in the group.
+   *   2. Changes to the set of participants who have subscribed the the barrier
+   */
   public void onProcessorChange(List<String> processors) {
-    LOG.info("ZkJobCoordinator::onProcessorChange - list of processors changed! List size=" + processors.size());
-    debounceTimer.scheduleAfterDebounceTime(ON_PROCESSOR_CHANGE, debounceTimeMs,
-        () -> doOnProcessorChange(processors));
+    if (leaderElector.amILeader()) {
+      LOG.info("ZkJobCoordinator::onProcessorChange - list of processors changed. List size=" + processors.size());
+      debounceTimer.scheduleAfterDebounceTime(ON_PROCESSOR_CHANGE, debounceTimeMs, () -> doOnProcessorChange(processors));
+    }
   }
 
   void doOnProcessorChange(List<String> processors) {
     // if list of processors is empty - it means we are called from 'onBecomeLeader'
     // TODO: Handle empty currentProcessorIds.
-    List<String> currentProcessorIds = getActualProcessorIds(processors);
-    Set<String> uniqueProcessorIds = new HashSet<String>(currentProcessorIds);
+    List<String> currentProcessorIds = zkUtils.getSortedActiveProcessorsIDs();
+    Set<String> uniqueProcessorIds = new HashSet<>(currentProcessorIds);
 
     if (currentProcessorIds.size() != uniqueProcessorIds.size()) {
       LOG.info("Processors: {} has duplicates. Not generating JobModel.", currentProcessorIds);
@@ -269,42 +283,6 @@ public class ZkJobCoordinator implements JobCoordinator, ZkControllerListener {
     debounceTimer.scheduleAfterDebounceTime(ON_ZK_CLEANUP, 0, () -> zkUtils.cleanupZK(NUM_VERSIONS_TO_LEAVE));
   }
 
-  @Override
-  public void onNewJobModelAvailable(final String version) {
-    debounceTimer.scheduleAfterDebounceTime(JOB_MODEL_VERSION_CHANGE, 0, () ->
-      {
-        LOG.info("pid=" + processorId + ": new JobModel available");
-        // get the new job model from ZK
-        newJobModel = zkUtils.getJobModel(version);
-        LOG.info("pid=" + processorId + ": new JobModel available. ver=" + version + "; jm = " + newJobModel);
-
-        if (!newJobModel.getContainers().containsKey(processorId)) {
-          LOG.info("New JobModel does not contain pid={}. Stopping this processor. New JobModel: {}",
-              processorId, newJobModel);
-          stop();
-        } else {
-          // stop current work
-          if (coordinatorListener != null) {
-            coordinatorListener.onJobModelExpired();
-          }
-          // update ZK and wait for all the processors to get this new version
-          barrier.join(version, processorId);
-        }
-      });
-  }
-
-  @Override
-  public void onNewJobModelConfirmed(String version) {
-    LOG.info("pid=" + processorId + "new version " + version + " of the job model got confirmed");
-    // get the new Model
-    JobModel jobModel = getJobModel();
-
-    // start the container with the new model
-    if (coordinatorListener != null) {
-      coordinatorListener.onNewJobModel(processorId, jobModel);
-    }
-  }
-
   private String createProcessorId(Config config) {
     // TODO: This check to be removed after 0.13+
     ApplicationConfig appConfig = new ApplicationConfig(config);
@@ -321,17 +299,6 @@ public class ZkJobCoordinator implements JobCoordinator, ZkControllerListener {
     }
   }
 
-  private List<String> getActualProcessorIds(List<String> processors) {
-    if (processors.size() > 0) {
-      // we should use this list
-      // but it needs to be converted into PIDs, which is part of the data
-      return zkUtils.getActiveProcessorsIDs(processors);
-    } else {
-      // get the current list of processors
-      return zkUtils.getSortedActiveProcessorsIDs();
-    }
-  }
-
   /**
    * Generate new JobModel when becoming a leader or the list of processor changed.
    */
@@ -350,18 +317,16 @@ public class ZkJobCoordinator implements JobCoordinator, ZkControllerListener {
      * to host mapping) is passed in as null when building the jobModel.
      */
     JobModel model = JobModelManager.readJobModel(this.config, changeLogPartitionMap, null, streamMetadataCache, processors);
-    // Nuke the configuration in JobModel.
     return new JobModel(new MapConfig(), model.getContainers());
   }
 
   class LeaderElectorListenerImpl implements LeaderElectorListener {
     @Override
     public void onBecomingLeader() {
-      LOG.info("ZkJobCoordinator::onBecomeLeader - I became the leader!");
+      LOG.info("ZkJobCoordinator::onBecomeLeader - I became the leader");
       metrics.isLeader.set(true);
-      zkController.subscribeToProcessorChange();
-      debounceTimer.scheduleAfterDebounceTime(ON_PROCESSOR_CHANGE, debounceTimeMs, () ->
-        {
+      zkUtils.subscribeToProcessorChange(new ProcessorChangeHandler(zkUtils));
+      debounceTimer.scheduleAfterDebounceTime(ON_PROCESSOR_CHANGE, debounceTimeMs, () -> {
           // actual actions to do are the same as onProcessorChange
           doOnProcessorChange(new ArrayList<>());
         });
@@ -379,11 +344,9 @@ public class ZkJobCoordinator implements JobCoordinator, ZkControllerListener {
       startTime = System.nanoTime();
 
       metrics.barrierCreation.inc();
-      debounceTimer.scheduleAfterDebounceTime(
-          barrierAction,
-        (new ZkConfig(config)).getZkBarrierTimeoutMs(),
-        () -> barrier.expire(version)
-      );
+      if (leaderElector.amILeader()) {
+        debounceTimer.scheduleAfterDebounceTime(barrierAction, (new ZkConfig(config)).getZkBarrierTimeoutMs(), () -> barrier.expire(version));
+      }
     }
 
     public void onBarrierStateChanged(final String version, ZkBarrierForVersionUpgrade.State state) {
@@ -391,16 +354,24 @@ public class ZkJobCoordinator implements JobCoordinator, ZkControllerListener {
       metrics.barrierStateChange.inc();
       metrics.singleBarrierRebalancingTime.update(System.nanoTime() - startTime);
       if (ZkBarrierForVersionUpgrade.State.DONE.equals(state)) {
-        debounceTimer.scheduleAfterDebounceTime(barrierAction, 0, () -> onNewJobModelConfirmed(version));
+        debounceTimer.scheduleAfterDebounceTime(barrierAction, 0, () -> {
+            LOG.info("pid=" + processorId + "new version " + version + " of the job model got confirmed");
+
+            // read the new Model
+            JobModel jobModel = getJobModel();
+            // start the container with the new model
+            if (coordinatorListener != null) {
+              coordinatorListener.onNewJobModel(processorId, jobModel);
+            }
+          });
       } else {
         if (ZkBarrierForVersionUpgrade.State.TIMED_OUT.equals(state)) {
           // no-op for non-leaders
           // for leader: make sure we do not stop - so generate a new job model
           LOG.warn("Barrier for version " + version + " timed out.");
-          if (zkController.isLeader()) {
+          if (leaderElector.amILeader()) {
             LOG.info("Leader will schedule a new job model generation");
-            debounceTimer.scheduleAfterDebounceTime(ON_PROCESSOR_CHANGE, debounceTimeMs, () ->
-              {
+            debounceTimer.scheduleAfterDebounceTime(ON_PROCESSOR_CHANGE, debounceTimeMs, () -> {
                 // actual actions to do are the same as onProcessorChange
                 doOnProcessorChange(new ArrayList<>());
               });
@@ -417,14 +388,82 @@ public class ZkJobCoordinator implements JobCoordinator, ZkControllerListener {
     }
   }
 
+  class ProcessorChangeHandler extends ZkUtils.GenerationAwareZkChildListener {
+
+    public ProcessorChangeHandler(ZkUtils zkUtils) {
+      super(zkUtils, "ProcessorChangeHandler");
+    }
+
+    /**
+     * Called when the children of the given path changed.
+     *
+     * @param parentPath      The parent path
+     * @param currentChildren The children or null if the root node (parent path) was deleted.
+     * @throws Exception
+     */
+    @Override
+    public void doHandleChildChange(String parentPath, List<String> currentChildren)
+        throws Exception {
+      if (currentChildren == null) {
+        LOG.info("handleChildChange on path " + parentPath + " was invoked with NULL list of children");
+      } else {
+        LOG.info("ProcessorChangeHandler::handleChildChange - Path: {} Current Children: {} ", parentPath, currentChildren);
+        onProcessorChange(currentChildren);
+      }
+    }
+  }
+
+  class ZkJobModelVersionChangeHandler extends ZkUtils.GenerationAwareZkDataListener {
+
+    public ZkJobModelVersionChangeHandler(ZkUtils zkUtils) {
+      super(zkUtils, "ZkJobModelVersionChangeHandler");
+    }
+
+    /**
+     * Invoked when there is a change to the JobModelVersion z-node. It signifies that a new JobModel version is available.
+     */
+    @Override
+    public void doHandleDataChange(String dataPath, Object data) {
+      debounceTimer.scheduleAfterDebounceTime(JOB_MODEL_VERSION_CHANGE, 0, () -> {
+          String jobModelVersion = (String) data;
+
+          LOG.info("Got a notification for new JobModel version. Path = {} Version = {}", dataPath, data);
+
+          newJobModel = zkUtils.getJobModel(jobModelVersion);
+          LOG.info("pid=" + processorId + ": new JobModel is available. Version =" + jobModelVersion + "; JobModel = " + newJobModel);
+
+          if (!newJobModel.getContainers().containsKey(processorId)) {
+            LOG.info("New JobModel does not contain pid={}. Stopping this processor. New JobModel: {}",
+                processorId, newJobModel);
+            stop();
+          } else {
+            // stop current work
+            if (coordinatorListener != null) {
+              coordinatorListener.onJobModelExpired();
+            }
+            // update ZK and wait for all the processors to get this new version
+            barrier.join(jobModelVersion, processorId);
+          }
+        });
+    }
+
+    @Override
+    public void doHandleDataDeleted(String dataPath) {
+      LOG.warn("JobModel version z-node has been deleted. Shutting down the coordinator" + dataPath);
+      debounceTimer.scheduleAfterDebounceTime("JOB_MODEL_VERSION_DELETED", 0,  () -> stop());
+    }
+  }
+
+
   /// listener to handle ZK state change events
+  @VisibleForTesting
   class ZkSessionStateChangedListener implements IZkStateListener {
 
     private static final String ZK_SESSION_ERROR = "ZK_SESSION_ERROR";
+    private static final String ZK_SESSION_EXPIRED = "ZK_SESSION_EXPIRED";
 
     @Override
-    public void handleStateChanged(Watcher.Event.KeeperState state)
-        throws Exception {
+    public void handleStateChanged(Watcher.Event.KeeperState state) {
       switch (state) {
         case Expired:
           // if the session has expired it means that all the registration's ephemeral nodes are gone.
@@ -433,12 +472,26 @@ public class ZkJobCoordinator implements JobCoordinator, ZkControllerListener {
           // increase generation of the ZK session. All the callbacks from the previous generation will be ignored.
           zkUtils.incGeneration();
 
-          if (coordinatorListener != null) {
-            coordinatorListener.onJobModelExpired();
-          }
-
           // reset all the values that might have been from the previous session (e.g ephemeral node path)
           zkUtils.unregister();
+          if (leaderElector.amILeader()) {
+            leaderElector.resignLeadership();
+          }
+          /**
+           * After this event, one amongst the following two things could potentially happen:
+           * A. On successful reconnect to another zookeeper server in ensemble, this processor is going to
+           * join the group again as new processor. In this case, retaining buffered events in debounceTimer will be unnecessary.
+           * B. If zookeeper server is unreachable, handleSessionEstablishmentError callback will be triggered indicating
+           * a error scenario. In this case, retaining buffered events in debounceTimer will be unnecessary.
+           */
+          LOG.info("Cancelling all scheduled actions in session expiration for processorId: {}.", processorId);
+          debounceTimer.cancelAllScheduledActions();
+          debounceTimer.scheduleAfterDebounceTime(ZK_SESSION_EXPIRED, 0, () -> {
+              if (coordinatorListener != null) {
+                coordinatorListener.onJobModelExpired();
+              }
+            });
+
           return;
         case Disconnected:
           // if the session has expired it means that all the registration's ephemeral nodes are gone.
@@ -460,21 +513,21 @@ public class ZkJobCoordinator implements JobCoordinator, ZkControllerListener {
         default:
           // received SyncConnected, ConnectedReadOnly, and SaslAuthenticated. NoOp
           LOG.info("Got ZK event " + state.toString() + " for processor=" + processorId + ". Continue");
+          return;
       }
     }
 
     @Override
-    public void handleNewSession()
-        throws Exception {
+    public void handleNewSession() {
       LOG.info("Got new session created event for processor=" + processorId);
-
+      debounceTimer.cancelAllScheduledActions();
       LOG.info("register zk controller for the new session");
-      zkController.register();
+      leaderElector.tryBecomeLeader();
+      zkUtils.subscribeToJobModelVersionChange(new ZkJobModelVersionChangeHandler(zkUtils));
     }
 
     @Override
-    public void handleSessionEstablishmentError(Throwable error)
-        throws Exception {
+    public void handleSessionEstablishmentError(Throwable error) {
       // this means we cannot connect to zookeeper to establish a session
       LOG.info("handleSessionEstablishmentError received for processor=" + processorId, error);
       debounceTimer.scheduleAfterDebounceTime(ZK_SESSION_ERROR, 0, () -> stop());
