@@ -21,12 +21,15 @@ package org.apache.samza.container
 
 import java.io.File
 import java.lang.management.ManagementFactory
+import java.lang.reflect.InvocationTargetException
 import java.net.{URL, UnknownHostException}
 import java.nio.file.Path
+import java.time.Duration
 import java.util
 import java.util.Base64
 import java.util.concurrent.{ExecutorService, Executors, ScheduledExecutorService, TimeUnit}
 
+import com.google.common.annotations.VisibleForTesting
 import com.google.common.util.concurrent.ThreadFactoryBuilder
 import org.apache.samza.checkpoint.{CheckpointListener, CheckpointManagerFactory, OffsetManager, OffsetManagerMetrics}
 import org.apache.samza.config.JobConfig.Config2Job
@@ -40,18 +43,18 @@ import org.apache.samza.config._
 import org.apache.samza.container.disk.DiskSpaceMonitor.Listener
 import org.apache.samza.container.disk.{DiskQuotaPolicyFactory, DiskSpaceMonitor, NoThrottlingDiskQuotaPolicyFactory, PollingScanDiskSpaceMonitor}
 import org.apache.samza.container.host.{StatisticsMonitorImpl, SystemMemoryStatistics, SystemStatisticsMonitor}
-import org.apache.samza.coordinator.stream.{CoordinatorStreamManager, CoordinatorStreamSystemProducer}
-import org.apache.samza.job.model.JobModel
+import org.apache.samza.context._
+import org.apache.samza.job.model.{ContainerModel, JobModel}
 import org.apache.samza.metrics.{JmxServer, JvmMetrics, MetricsRegistryMap, MetricsReporter}
 import org.apache.samza.serializers._
 import org.apache.samza.serializers.model.SamzaObjectMapper
-import org.apache.samza.storage.{StorageEngineFactory, TaskStorageManager}
+import org.apache.samza.storage._
 import org.apache.samza.system._
 import org.apache.samza.system.chooser.{DefaultChooser, MessageChooserFactory, RoundRobinChooserFactory}
 import org.apache.samza.table.TableManager
+import org.apache.samza.table.utils.SerdeUtils
 import org.apache.samza.task._
-import org.apache.samza.util.Util
-import org.apache.samza.util._
+import org.apache.samza.util.{Util, _}
 import org.apache.samza.{SamzaContainerStatus, SamzaException}
 
 import scala.collection.JavaConverters._
@@ -99,7 +102,7 @@ object SamzaContainer extends Logging {
     if(System.getenv(ShellCommandConfig.ENV_LOGGED_STORE_BASE_DIR) != null) {
       val jobNameAndId = (
         config.getName.getOrElse(throw new ConfigException("Missing required config: job.name")),
-        config.getJobId.getOrElse("1")
+        config.getJobId
       )
 
       loggedStorageBaseDir = new File(System.getenv(ShellCommandConfig.ENV_LOGGED_STORE_BASE_DIR)
@@ -120,22 +123,17 @@ object SamzaContainer extends Logging {
   def apply(
     containerId: String,
     jobModel: JobModel,
-    config: Config,
     customReporters: Map[String, MetricsReporter] = Map[String, MetricsReporter](),
-    taskFactory: Object) = {
+    taskFactory: TaskFactory[_],
+    jobContext: JobContext,
+    applicationContainerContextFactoryOption: Option[ApplicationContainerContextFactory[ApplicationContainerContext]],
+    applicationTaskContextFactoryOption: Option[ApplicationTaskContextFactory[ApplicationTaskContext]]
+  ) = {
+    val config = jobContext.getConfig
     val containerModel = jobModel.getContainers.get(containerId)
     val containerName = "samza-container-%s" format containerId
     val maxChangeLogStreamPartitions = jobModel.maxChangeLogStreamPartitions
 
-    var coordinatorStreamManager: CoordinatorStreamManager = null
-    var localityManager: LocalityManager = null
-    if (new ClusterManagerConfig(config).getHostAffinityEnabled()) {
-      val registryMap = new MetricsRegistryMap(containerName)
-      val coordinatorStreamSystemProducer = new CoordinatorStreamSystemProducer(config, new SamzaContainerMetrics(containerName, registryMap).registry)
-      coordinatorStreamManager = new CoordinatorStreamManager(coordinatorStreamSystemProducer)
-      localityManager = new LocalityManager(coordinatorStreamManager)
-    }
-
     val containerPID = ManagementFactory.getRuntimeMXBean().getName()
 
     info("Setting up Samza container: %s" format containerName)
@@ -192,7 +190,7 @@ object SamzaContainer extends Logging {
     info("Got system factories: %s" format systemFactories.keys)
 
     val systemAdmins = new SystemAdmins(config)
-    info("Got system admins: %s" format systemAdmins.getSystemAdminsMap().keySet())
+    info("Got system admins: %s" format systemAdmins.getSystemNames)
 
     val streamMetadataCache = new StreamMetadataCache(systemAdmins)
     val inputStreamMetadata = streamMetadataCache.getStreamMetadata(inputSystemStreams)
@@ -323,10 +321,27 @@ object SamzaContainer extends Logging {
       .getStoreNames
       .filter(config.getChangelogStream(_).isDefined)
       .map(name => (name, config.getChangelogStream(name).get)).toMap
-      .mapValues(Util.getSystemStreamFromNames(_))
+      .mapValues(StreamUtil.getSystemStreamFromNames(_))
 
     info("Got change log system streams: %s" format changeLogSystemStreams)
 
+    /*
+     * This keeps track of the changelog SSPs that are associated with the whole container. This is used so that we can
+     * prefetch the metadata about the all of the changelog SSPs associated with the container whenever we need the
+     * metadata about some of the changelog SSPs.
+     * An example use case is when Samza writes offset files for stores ({@link TaskStorageManager}). Each task is
+     * responsible for its own offset file, but if we can do prefetching, then most tasks will already have cached
+     * metadata by the time they need the offset metadata.
+     * Note: By using all changelog streams to build the sspsToPrefetch, any fetches done for persisted stores will
+     * include the ssps for non-persisted stores, so this is slightly suboptimal. However, this does not increase the
+     * actual number of calls to the {@link SystemAdmin}, and we can decouple this logic from the per-task objects (e.g.
+     * {@link TaskStorageManager}).
+     */
+    val changelogSSPMetadataCache = new SSPMetadataCache(systemAdmins,
+      Duration.ofSeconds(5),
+      SystemClock.instance,
+      getChangelogSSPsForContainer(containerModel, changeLogSystemStreams).asJava)
+
     val intermediateStreams = config
       .getStreamIds
       .filter(config.getIsIntermediateStream(_))
@@ -334,6 +349,14 @@ object SamzaContainer extends Logging {
 
     info("Got intermediate streams: %s" format intermediateStreams)
 
+    val sideInputStoresToSystemStreams = config.getStoreNames
+      .map { storeName => (storeName, config.getSideInputs(storeName)) }
+      .filter { case (storeName, sideInputs) => sideInputs.nonEmpty }
+      .map { case (storeName, sideInputs) => (storeName, sideInputs.map(StreamUtil.getSystemStreamFromNameOrId(config, _))) }
+      .toMap
+
+    info("Got side input store system streams: %s" format sideInputStoresToSystemStreams)
+
     val controlMessageKeySerdes = intermediateStreams
       .flatMap(streamId => {
         val systemStream = config.streamIdToSystemStream(streamId)
@@ -470,8 +493,10 @@ object SamzaContainer extends Logging {
       .asScala
       .map(_.getTaskName)
       .toSet
-    val containerContext = new SamzaContainerContext(containerId, config, taskNames.asJava, samzaContainerMetrics.registry)
 
+    val containerContext = new ContainerContextImpl(containerModel, samzaContainerMetrics.registry)
+    val applicationContainerContextOption = applicationContainerContextFactoryOption
+      .map(_.create(jobContext, containerContext))
 
     val storeWatchPaths = new util.HashSet[Path]()
 
@@ -511,7 +536,7 @@ object SamzaContainer extends Logging {
       val nonLoggedStorageBaseDir = getNonLoggedStorageBaseDir(config, defaultStoreBaseDir)
       info("Got base directory for non logged data stores: %s" format nonLoggedStorageBaseDir)
 
-      var loggedStorageBaseDir = getLoggedStorageBaseDir(config, defaultStoreBaseDir)
+      val loggedStorageBaseDir = getLoggedStorageBaseDir(config, defaultStoreBaseDir)
       info("Got base directory for logged data stores: %s" format loggedStorageBaseDir)
 
       val taskStores = storageEngineFactories
@@ -535,7 +560,9 @@ object SamzaContainer extends Logging {
               case _ => null
             }
 
-            val storeDir = if (changeLogSystemStreamPartition != null) {
+            // We use the logged storage base directory for change logged and side input stores since side input stores
+            // dont have changelog configured.
+            val storeDir = if (changeLogSystemStreamPartition != null || sideInputStoresToSystemStreams.contains(storeName)) {
               TaskStorageManager.getStorePartitionDir(loggedStorageBaseDir, storeName, taskName)
             } else {
               TaskStorageManager.getStorePartitionDir(nonLoggedStorageBaseDir, storeName, taskName)
@@ -551,19 +578,44 @@ object SamzaContainer extends Logging {
               collector,
               taskInstanceMetrics.registry,
               changeLogSystemStreamPartition,
+              jobContext,
               containerContext)
             (storeName, storageEngine)
         }
 
       info("Got task stores: %s" format taskStores)
 
+      val taskSSPs = taskModel.getSystemStreamPartitions.asScala.toSet
+      info("Got task SSPs: %s" format taskSSPs)
+
+      val (sideInputStores, nonSideInputStores) =
+        taskStores.partition { case (storeName, _) => sideInputStoresToSystemStreams.contains(storeName)}
+
+      val sideInputStoresToSSPs = sideInputStoresToSystemStreams.mapValues(sideInputSystemStreams =>
+        taskSSPs.filter(ssp => sideInputSystemStreams.contains(ssp.getSystemStream)).asJava)
+
+      val taskSideInputSSPs = sideInputStoresToSSPs.values.flatMap(_.asScala).toSet
+
+      info ("Got task side input SSPs: %s" format taskSideInputSSPs)
+
+      val sideInputStoresToProcessor = sideInputStores.keys.map(storeName => {
+          // serialized instances takes precedence over the factory configuration.
+          config.getSideInputsProcessorSerializedInstance(storeName).map(serializedInstance =>
+              (storeName, SerdeUtils.deserialize("Side Inputs Processor", serializedInstance)))
+            .orElse(config.getSideInputsProcessorFactory(storeName).map(factoryClassName =>
+              (storeName, Util.getObj(factoryClassName, classOf[SideInputsProcessorFactory])
+                .getSideInputsProcessor(config, taskInstanceMetrics.registry))))
+            .get
+        }).toMap
+
       val storageManager = new TaskStorageManager(
         taskName = taskName,
-        taskStores = taskStores,
+        taskStores = nonSideInputStores,
         storeConsumers = storeConsumers,
         changeLogSystemStreams = changeLogSystemStreams,
         maxChangeLogStreamPartitions,
         streamMetadataCache = streamMetadataCache,
+        sspMetadataCache = changelogSSPMetadataCache,
         nonLoggedStoreBaseDir = nonLoggedStorageBaseDir,
         loggedStoreBaseDir = loggedStorageBaseDir,
         partition = taskModel.getChangelogPartition,
@@ -571,35 +623,46 @@ object SamzaContainer extends Logging {
         new StorageConfig(config).getChangeLogDeleteRetentionsInMs,
         new SystemClock)
 
+      var sideInputStorageManager: TaskSideInputStorageManager = null
+      if (sideInputStores.nonEmpty) {
+        sideInputStorageManager = new TaskSideInputStorageManager(
+          taskName,
+          streamMetadataCache,
+          loggedStorageBaseDir.getPath,
+          sideInputStores.asJava,
+          sideInputStoresToProcessor.asJava,
+          sideInputStoresToSSPs.asJava,
+          systemAdmins,
+          config,
+          new SystemClock)
+      }
+
       val tableManager = new TableManager(config, serdes.asJava)
 
       info("Got table manager")
 
-      val systemStreamPartitions = taskModel
-        .getSystemStreamPartitions
-        .asScala
-        .toSet
-
-      info("Retrieved SystemStreamPartitions " + systemStreamPartitions + " for " + taskName)
-
       def createTaskInstance(task: Any): TaskInstance = new TaskInstance(
           task = task,
-          taskName = taskName,
-          config = config,
+          taskModel = taskModel,
           metrics = taskInstanceMetrics,
           systemAdmins = systemAdmins,
           consumerMultiplexer = consumerMultiplexer,
           collector = collector,
-          containerContext = containerContext,
           offsetManager = offsetManager,
           storageManager = storageManager,
           tableManager = tableManager,
           reporters = reporters,
-          systemStreamPartitions = systemStreamPartitions,
+          systemStreamPartitions = taskSSPs,
           exceptionHandler = TaskInstanceExceptionHandler(taskInstanceMetrics, config),
           jobModel = jobModel,
           streamMetadataCache = streamMetadataCache,
-          timerExecutor = timerExecutor)
+          timerExecutor = timerExecutor,
+          sideInputSSPs = taskSideInputSSPs,
+          sideInputStorageManager = sideInputStorageManager,
+          jobContext = jobContext,
+          containerContext = containerContext,
+          applicationContainerContextOption = applicationContainerContextOption,
+          applicationTaskContextFactoryOption = applicationTaskContextFactoryOption)
 
       val taskInstance = createTaskInstance(task)
 
@@ -655,15 +718,13 @@ object SamzaContainer extends Logging {
     info("Samza container setup complete.")
 
     new SamzaContainer(
-      containerContext = containerContext,
+      config = config,
       taskInstances = taskInstances,
       runLoop = runLoop,
       systemAdmins = systemAdmins,
       consumerMultiplexer = consumerMultiplexer,
       producerMultiplexer = producerMultiplexer,
       offsetManager = offsetManager,
-      coordinatorStreamManager = coordinatorStreamManager,
-      localityManager = localityManager,
       securityManager = securityManager,
       metrics = samzaContainerMetrics,
       reporters = reporters,
@@ -671,12 +732,27 @@ object SamzaContainer extends Logging {
       diskSpaceMonitor = diskSpaceMonitor,
       hostStatisticsMonitor = memoryStatisticsMonitor,
       taskThreadPool = taskThreadPool,
-      timerExecutor = timerExecutor)
+      timerExecutor = timerExecutor,
+      containerContext = containerContext,
+      applicationContainerContextOption = applicationContainerContextOption)
+  }
+
+  /**
+    * Builds the set of SSPs for all changelogs on this container.
+    */
+  @VisibleForTesting
+  private[container] def getChangelogSSPsForContainer(containerModel: ContainerModel,
+    changeLogSystemStreams: Map[String, SystemStream]): Set[SystemStreamPartition] = {
+    containerModel.getTasks.values().asScala
+      .map(taskModel => taskModel.getChangelogPartition)
+      .flatMap(changelogPartition => changeLogSystemStreams.map { case (_, systemStream) =>
+        new SystemStreamPartition(systemStream, changelogPartition) })
+      .toSet
   }
 }
 
 class SamzaContainer(
-  containerContext: SamzaContainerContext,
+  config: Config,
   taskInstances: Map[TaskName, TaskInstance],
   runLoop: Runnable,
   systemAdmins: SystemAdmins,
@@ -686,22 +762,22 @@ class SamzaContainer(
   diskSpaceMonitor: DiskSpaceMonitor = null,
   hostStatisticsMonitor: SystemStatisticsMonitor = null,
   offsetManager: OffsetManager = new OffsetManager,
-  coordinatorStreamManager: CoordinatorStreamManager = null,
   localityManager: LocalityManager = null,
   securityManager: SecurityManager = null,
   reporters: Map[String, MetricsReporter] = Map(),
   jvm: JvmMetrics = null,
   taskThreadPool: ExecutorService = null,
-  timerExecutor: ScheduledExecutorService = Executors.newSingleThreadScheduledExecutor) extends Runnable with Logging {
+  timerExecutor: ScheduledExecutorService = Executors.newSingleThreadScheduledExecutor,
+  containerContext: ContainerContext,
+  applicationContainerContextOption: Option[ApplicationContainerContext]) extends Runnable with Logging {
 
-  val shutdownMs = containerContext.config.getShutdownMs.getOrElse(TaskConfigJava.DEFAULT_TASK_SHUTDOWN_MS)
+  val shutdownMs = config.getShutdownMs.getOrElse(TaskConfigJava.DEFAULT_TASK_SHUTDOWN_MS)
   var shutdownHookThread: Thread = null
   var jmxServer: JmxServer = null
-  val isAutoCommitEnabled = containerContext.config.isAutoCommitEnabled
+  val isAutoCommitEnabled = config.isAutoCommitEnabled
 
   @volatile private var status = SamzaContainerStatus.NOT_STARTED
   private var exceptionSeen: Throwable = null
-  private var paused: Boolean = false
   private var containerListener: SamzaContainerListener = null
 
   def getStatus(): SamzaContainerStatus = status
@@ -712,19 +788,28 @@ class SamzaContainer(
     containerListener = listener
   }
 
+  def hasStopped(): Boolean = status == SamzaContainerStatus.STOPPED || status == SamzaContainerStatus.FAILED
+
   def run {
     try {
       info("Starting container.")
 
+      if (containerListener != null) {
+        containerListener.beforeStart()
+      }
+
       val startTime = System.nanoTime()
       status = SamzaContainerStatus.STARTING
-
-      jmxServer = new JmxServer()
+      if (config.getJMXEnabled) {
+        jmxServer = new JmxServer()
+      }
+      applicationContainerContextOption.foreach(_.start)
 
       startMetrics
+      startDiagnostics
       startAdmins
       startOffsetManager
-      startLocalityManager
+      storeContainerLocality
       startStores
       startTableManager
       startDiskSpaceMonitor
@@ -738,7 +823,7 @@ class SamzaContainer(
       info("Entering run loop.")
       status = SamzaContainerStatus.STARTED
       if (containerListener != null) {
-        containerListener.onContainerStart()
+        containerListener.afterStart()
       }
       metrics.containerStartupTime.update(System.nanoTime() - startTime)
       runLoop.run
@@ -754,10 +839,11 @@ class SamzaContainer(
     }
 
     try {
-      info("Shutting down.")
+      info("Shutting down SamzaContainer.")
       removeShutdownHook
-
-      jmxServer.stop
+      if (jmxServer != null) {
+        jmxServer.stop
+      }
 
       shutdownConsumers
       shutdownTask
@@ -766,12 +852,13 @@ class SamzaContainer(
       shutdownDiskSpaceMonitor
       shutdownHostStatisticsMonitor
       shutdownProducers
-      shutdownLocalityManager
       shutdownOffsetManager
       shutdownMetrics
       shutdownSecurityManger
       shutdownAdmins
 
+      applicationContainerContextOption.foreach(_.stop)
+
       if (!status.equals(SamzaContainerStatus.FAILED)) {
         status = SamzaContainerStatus.STOPPED
       }
@@ -789,26 +876,15 @@ class SamzaContainer(
     status match {
       case SamzaContainerStatus.STOPPED =>
         if (containerListener != null) {
-          containerListener.onContainerStop(paused)
+          containerListener.afterStop()
         }
       case SamzaContainerStatus.FAILED =>
         if (containerListener != null) {
-          containerListener.onContainerFailed(exceptionSeen)
+          containerListener.afterFailure(exceptionSeen)
         }
     }
   }
 
-  // TODO: We want to introduce a "PAUSED" state for SamzaContainer in the future so that StreamProcessor can pause and
-  // unpause the container when the jobmodel changes.
-  /**
-   * Marks the [[SamzaContainer]] as being paused by the called due to a change in [[JobModel]] and then, asynchronously
-   * shuts down this [[SamzaContainer]]
-   */
-  def pause(): Unit = {
-    paused = true
-    shutdown()
-  }
-
   /**
    * <p>
    *   Asynchronously shuts down this [[SamzaContainer]]
@@ -816,15 +892,17 @@ class SamzaContainer(
    * <br>
    * <b>Implementation</b>: Stops the [[RunLoop]], which will eventually transition the container from
    * [[SamzaContainerStatus.STARTED]] to either [[SamzaContainerStatus.STOPPED]] or [[SamzaContainerStatus.FAILED]]].
-   * Based on the final `status`, [[SamzaContainerListener#onContainerStop(boolean)]] or
-   * [[SamzaContainerListener#onContainerFailed(Throwable)]] will be invoked respectively.
+   * Based on the final `status`, [[SamzaContainerListener#afterStop()]] or
+    * [[SamzaContainerListener#afterFailure(Throwable]] will be invoked respectively.
    *
    * @throws SamzaException, Thrown when the container has already been stopped or failed
    */
   def shutdown(): Unit = {
-    if (status == SamzaContainerStatus.STOPPED || status == SamzaContainerStatus.FAILED) {
-      throw new IllegalContainerStateException("Cannot shutdown a container with status " + status)
+    if (status == SamzaContainerStatus.FAILED || status == SamzaContainerStatus.STOPPED) {
+      warn("Shutdown is no-op since the container is already in state: " + status)
+      return
     }
+
     shutdownRunLoop()
   }
 
@@ -869,6 +947,24 @@ class SamzaContainer(
     })
   }
 
+  def startDiagnostics {
+    if (config.getDiagnosticsEnabled) {
+      info("Starting diagnostics.")
+
+      try {
+        val diagnosticsAppender = Class.forName(config.getDiagnosticsAppenderClass).
+          getDeclaredConstructor(classOf[SamzaContainerMetrics]).newInstance(this.metrics);
+      }
+      catch {
+        case e@(_: ClassNotFoundException | _: InstantiationException | _: InvocationTargetException) => {
+          error("Failed to instantiate diagnostic appender", e)
+          throw new ConfigException("Failed to instantiate diagnostic appender class " +
+            config.getDiagnosticsAppenderClass, e)
+        }
+      }
+    }
+  }
+
   def startOffsetManager {
     info("Registering task instances with offsets.")
 
@@ -879,31 +975,29 @@ class SamzaContainer(
     offsetManager.start
   }
 
-  def startLocalityManager {
-    if(localityManager != null) {
-      if(coordinatorStreamManager == null) {
-        // This should never happen.
-        throw new IllegalStateException("Cannot start LocalityManager without a CoordinatorStreamManager")
-      }
-
-      val containerName = "SamzaContainer-" + String.valueOf(containerContext.id)
-      info("Registering %s with the coordinator stream manager." format containerName)
-      coordinatorStreamManager.start
-      coordinatorStreamManager.register(containerName)
-
-      info("Writing container locality and JMX address to Coordinator Stream")
+  def storeContainerLocality {
+    val isHostAffinityEnabled: Boolean = new ClusterManagerConfig(config).getHostAffinityEnabled
+    if (isHostAffinityEnabled) {
+      val localityManager: LocalityManager = new LocalityManager(config, containerContext.getContainerMetricsRegistry)
+      val containerId = containerContext.getContainerModel.getId
+      val containerName = "SamzaContainer-" + containerId
+      info("Registering %s with metadata store" format containerName)
       try {
         val hostInet = Util.getLocalHost
         val jmxUrl = if (jmxServer != null) jmxServer.getJmxUrl else ""
         val jmxTunnelingUrl = if (jmxServer != null) jmxServer.getTunnelingJmxUrl else ""
-        localityManager.writeContainerToHostMapping(containerContext.id, hostInet.getHostName, jmxUrl, jmxTunnelingUrl)
+        info("Writing container locality and JMX address to metadata store")
+        localityManager.writeContainerToHostMapping(containerId, hostInet.getHostName)
       } catch {
         case uhe: UnknownHostException =>
           warn("Received UnknownHostException when persisting locality info for container %s: " +
-            "%s" format (containerContext.id, uhe.getMessage))  //No-op
+            "%s" format (containerId, uhe.getMessage))  //No-op
         case unknownException: Throwable =>
           warn("Received an exception when persisting locality info for container %s: " +
-            "%s" format (containerContext.id, unknownException.getMessage))
+            "%s" format (containerId, unknownException.getMessage))
+      } finally {
+        info("Shutting down locality manager.")
+        localityManager.close()
       }
     }
   }
@@ -941,7 +1035,6 @@ class SamzaContainer(
     systemAdmins.start
   }
 
-
   def startProducers {
     info("Registering task instances with producers.")
 
@@ -1017,7 +1110,6 @@ class SamzaContainer(
     systemAdmins.stop
   }
 
-
   def shutdownProducers {
     info("Shutting down producer multiplexer.")
 
@@ -1071,13 +1163,6 @@ class SamzaContainer(
     taskInstances.values.foreach(_.shutdownTableManager)
   }
 
-  def shutdownLocalityManager {
-    if(coordinatorStreamManager != null) {
-      info("Shutting down coordinator stream manager used by locality manager.")
-      coordinatorStreamManager.stop
-    }
-  }
-
   def shutdownOffsetManager {
     info("Shutting down offset manager.")
 
@@ -1117,15 +1202,4 @@ class SamzaContainer(
       hostStatisticsMonitor.stop()
     }
   }
-}
-
-/**
- * Exception thrown when the SamzaContainer tries to transition to an illegal state.
- * {@link SamzaContainerStatus} has more details on the state transitions.
- *
- * @param s String, Message associated with the exception
- * @param t Throwable, Wrapped error/exception thrown, if any.
- */
-class IllegalContainerStateException(s: String, t: Throwable) extends SamzaException(s, t) {
-  def this(s: String) = this(s, null)
-}
+}
\ No newline at end of file
