@@ -53,16 +53,20 @@ class ContainerProcessManagerMetrics(
     val mReleasedContainers = newGauge("released-containers", () => state.releasedContainers.get())
     val mContainers = newGauge("container-count", () => state.containerCount)
     val mRedundantNotifications = newGauge("redundant-notifications", () => state.redundantNotifications.get())
-
     val mJobHealthy = newGauge("job-healthy", () => if (state.jobHealthy.get()) 1 else 0)
-    val mLocalityMatchedRequests = newGauge(
-      "locality-matched",
-      () => {
-        if (state.containerRequests.get() != 0) {
-          state.matchedResourceRequests.get() / state.containerRequests.get()
-        } else {
-          0L
-        }
+    val mPreferredHostRequests = newGauge("preferred-host-requests", () => state.preferredHostRequests.get())
+    val mAnyHostRequests = newGauge("any-host-requests", () => state.anyHostRequests.get())
+    val mExpiredPreferredHostRequests = newGauge("expired-preferred-host-requests", () => state.expiredPreferredHostRequests.get())
+    val mExpiredAnyHostRequests = newGauge("expired-any-host-requests", () => state.expiredAnyHostRequests.get())
+
+    val mHostAffinityMatchPct = newGauge("host-affinity-match-pct", () => {
+      val numPreferredHostRequests = state.preferredHostRequests.get()
+      val numExpiredPreferredHostRequests = state.expiredPreferredHostRequests.get()
+      if (numPreferredHostRequests != 0) {
+            100.00 * (numPreferredHostRequests - numExpiredPreferredHostRequests) / numPreferredHostRequests
+          } else {
+            0L
+          }
       })
 
     jvm.start
