@@ -30,7 +30,6 @@ import org.junit.After;
 import org.junit.Before;
 import org.junit.Test;
 
-import java.io.IOException;
 import java.lang.reflect.Field;
 import java.util.HashMap;
 import java.util.Map;
@@ -202,58 +201,4 @@ public class TestContainerAllocator {
     listener.verify();
   }
 
-
-  /**
-   * If the container fails to start e.g because it fails to connect to a NM on a host that
-   * is down, the allocator should request a new container on a different host.
-   */
-  @Test
-  public void testRerequestOnAnyHostIfContainerStartFails() throws Exception {
-    final SamzaResource container = new SamzaResource(1, 1024, "2", "id0");
-    final SamzaResource container1 = new SamzaResource(1, 1024, "2", "id0");
-    manager.nextException = new IOException("Cant connect to RM");
-
-    // Set up our final asserts before starting the allocator thread
-    MockContainerListener listener = new MockContainerListener(2, 1, 2, 0, null, new Runnable() {
-      @Override
-      public void run() {
-        // The failed container should be released. The successful one should not.
-        assertNotNull(manager.releasedResources);
-        assertEquals(1, manager.releasedResources.size());
-        assertTrue(manager.releasedResources.contains(container));
-      }
-    },
-        new Runnable() {
-          @Override
-          public void run() {
-            // Test that the first request assignment had a preferred host and the retry didn't
-            assertEquals(2, requestState.assignedRequests.size());
-
-            SamzaResourceRequest request = requestState.assignedRequests.remove();
-            assertEquals("0", request.getContainerID());
-            assertEquals("2", request.getPreferredHost());
-
-            request = requestState.assignedRequests.remove();
-            assertEquals("0", request.getContainerID());
-            assertEquals("ANY_HOST", request.getPreferredHost());
-
-            // This routine should be called after the retry is assigned, but before it's started.
-            // So there should still be 1 container needed.
-            assertEquals(1, state.neededContainers.get());
-          }
-        }, null
-    );
-    state.neededContainers.set(1);
-    requestState.registerContainerListener(listener);
-
-    containerAllocator.requestResource("0", "2");
-    containerAllocator.addResource(container);
-    containerAllocator.addResource(container1);
-    allocatorThread.start();
-
-    listener.verify();
-
-  }
-
-
 }
