@@ -20,7 +20,7 @@ package org.apache.samza.storage.kv.inmemory
 
 import com.google.common.primitives.UnsignedBytes
 import org.apache.samza.util.Logging
-import org.apache.samza.storage.kv.{KeyValueStoreMetrics, KeyValueIterator, Entry, KeyValueStore}
+import org.apache.samza.storage.kv._
 import java.util
 
 /**
@@ -112,4 +112,16 @@ class InMemoryKeyValueStore(val metrics: KeyValueStoreMetrics = new KeyValueStor
     }
     found
   }
+
+  override def snapshot(from: Array[Byte], to: Array[Byte]): KeyValueSnapshot[Array[Byte], Array[Byte]] = {
+    // snapshot the underlying map
+    val entries = underlying.subMap(from, to).entrySet()
+    new KeyValueSnapshot[Array[Byte], Array[Byte]] {
+      override def iterator(): KeyValueIterator[Array[Byte], Array[Byte]] = {
+        new InMemoryIterator(entries.iterator())
+      }
+
+      override def close() { }
+    }
+  }
 }
