@@ -19,8 +19,6 @@
 
 package org.apache.samza.storage.kv
 
-import org.apache.samza.util.Util.notNull
-
 import scala.collection.JavaConverters._
 
 object NullSafeKeyValueStore {
@@ -85,4 +83,16 @@ class NullSafeKeyValueStore[K, V](store: KeyValueStore[K, V]) extends KeyValueSt
   def close {
     store.close
   }
+
+  private def notNull[T](obj: T, msg: String) = {
+    if (obj == null) {
+      throw new NullPointerException(msg)
+    }
+  }
+
+  override def snapshot(from: K, to: K): KeyValueSnapshot[K, V] = {
+    notNull(from, NullKeyErrorMessage)
+    notNull(to, NullKeyErrorMessage)
+    store.snapshot(from, to)
+  }
 }
