@@ -69,4 +69,8 @@ class MockKeyValueStore extends KeyValueStore[String, String] {
   override def flush() {}  // no-op
 
   override def close() { kvMap.clear() }
+
+  override def snapshot(from: String, to: String): KeyValueSnapshot[String, String] = {
+    throw new UnsupportedOperationException("iterator() not supported")
+  }
 }
\ No newline at end of file
