@@ -19,6 +19,7 @@
 
 package org.apache.samza.sql.avro;
 
+import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
@@ -29,17 +30,17 @@ import org.apache.avro.Schema;
 import org.apache.avro.generic.GenericData;
 import org.apache.avro.generic.GenericRecord;
 import org.apache.avro.generic.IndexedRecord;
+import org.apache.calcite.avatica.util.ByteString;
 import org.apache.samza.SamzaException;
 import org.apache.samza.config.Config;
 import org.apache.samza.operators.KV;
+import org.apache.samza.sql.SamzaSqlRelRecord;
 import org.apache.samza.sql.data.SamzaSqlRelMessage;
 import org.apache.samza.sql.interfaces.SamzaRelConverter;
 import org.apache.samza.system.SystemStream;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import static org.apache.samza.sql.data.SamzaSqlRelMessage.SamzaSqlRelRecord;
-
 
 /**
  * This class converts a Samza Avro messages to Relational messages and vice versa.
@@ -173,10 +174,12 @@ public class AvroRelConverter implements SamzaRelConverter {
                 getNonNullUnionSchema(schema).getValueType())));
       case UNION:
         return convertToAvroObject(relObj, getNonNullUnionSchema(schema));
-      case FIXED:
-        return new GenericData.Fixed(schema, ((String) relObj).getBytes());
       case ENUM:
         return new GenericData.EnumSymbol(schema, (String) relObj);
+      case FIXED:
+        return new GenericData.Fixed(schema, ((ByteString) relObj).getBytes());
+      case BYTES:
+        return ByteBuffer.wrap(((ByteString) relObj).getBytes());
       default:
         return relObj;
     }
@@ -219,8 +222,12 @@ public class AvroRelConverter implements SamzaRelConverter {
       case UNION:
         return convertToJavaObject(avroObj, getNonNullUnionSchema(schema));
       case ENUM:
-      case FIXED:
         return avroObj.toString();
+      case FIXED:
+        GenericData.Fixed fixed = (GenericData.Fixed) avroObj;
+        return new ByteString(fixed.bytes());
+      case BYTES:
+        return new ByteString(((ByteBuffer) avroObj).array());
 
       default:
         return avroObj;
