@@ -21,14 +21,13 @@ package org.apache.samza.sql.translator;
 
 import java.util.Arrays;
 import java.util.Collections;
-
-import java.util.List;
-import org.apache.calcite.rel.RelNode;
 import org.apache.calcite.rel.logical.LogicalFilter;
-import org.apache.calcite.rex.RexNode;
+import org.apache.samza.context.Context;
 import org.apache.samza.operators.MessageStream;
+import org.apache.samza.operators.functions.FilterFunction;
 import org.apache.samza.sql.data.Expression;
 import org.apache.samza.sql.data.SamzaSqlRelMessage;
+import org.apache.samza.sql.runner.SamzaSqlApplicationContext;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -40,20 +39,34 @@ import org.slf4j.LoggerFactory;
 class FilterTranslator {
 
   private static final Logger log = LoggerFactory.getLogger(FilterTranslator.class);
+  private final int queryId;
 
-  void translate(final LogicalFilter filter, final TranslatorContext context) {
-    MessageStream<SamzaSqlRelMessage> inputStream = context.getMessageStream(filter.getInput().getId());
-    MessageStream<SamzaSqlRelMessage> outputStream = translateFilter(inputStream, filter.getInputs(),
-        filter.getCondition(), context);
-    context.registerMessageStream(filter.getId(), outputStream);
+  FilterTranslator(int queryId) {
+    this.queryId = queryId;
   }
 
-  static MessageStream<SamzaSqlRelMessage> translateFilter(MessageStream<SamzaSqlRelMessage> inputStream,
-      List<RelNode> inputs, RexNode condition, final TranslatorContext context) {
-    Expression expr =
-        context.getExpressionCompiler().compile(inputs, Collections.singletonList(condition));
+  private static class FilterTranslatorFunction implements FilterFunction<SamzaSqlRelMessage> {
+    private transient Expression expr;
+    private transient TranslatorContext context;
+    private transient LogicalFilter filter;
+    private final int queryId;
 
-    return inputStream.filter(message -> {
+    private final int filterId;
+
+    FilterTranslatorFunction(int filterId, int queryId) {
+      this.filterId = filterId;
+      this.queryId = queryId;
+    }
+
+    @Override
+    public void init(Context context) {
+      this.context = ((SamzaSqlApplicationContext) context.getApplicationTaskContext()).getTranslatorContexts().get(queryId);
+      this.filter = (LogicalFilter) this.context.getRelNode(filterId);
+      this.expr = this.context.getExpressionCompiler().compile(filter.getInputs(), Collections.singletonList(filter.getCondition()));
+    }
+
+    @Override
+    public boolean apply(SamzaSqlRelMessage message) {
       Object[] result = new Object[1];
       expr.execute(context.getExecutionContext(), context.getDataContext(),
           message.getSamzaSqlRelRecord().getFieldValues().toArray(), result);
@@ -67,6 +80,16 @@ class FilterTranslator {
         log.error("return value is not boolean");
         return false;
       }
-    });
+    }
+  }
+
+  void translate(final LogicalFilter filter, final TranslatorContext context) {
+    MessageStream<SamzaSqlRelMessage> inputStream = context.getMessageStream(filter.getInput().getId());
+    final int filterId = filter.getId();
+
+    MessageStream<SamzaSqlRelMessage> outputStream = inputStream.filter(new FilterTranslatorFunction(filterId, queryId));
+
+    context.registerMessageStream(filterId, outputStream);
+    context.registerRelNode(filterId, filter);
   }
 }
