@@ -31,9 +31,12 @@ import org.apache.calcite.rex.RexInputRef;
 import org.apache.calcite.rex.RexNode;
 import org.apache.calcite.sql.validate.SqlUserDefinedFunction;
 import org.apache.samza.SamzaException;
+import org.apache.samza.context.Context;
 import org.apache.samza.operators.MessageStream;
+import org.apache.samza.operators.functions.MapFunction;
 import org.apache.samza.sql.data.Expression;
 import org.apache.samza.sql.data.SamzaSqlRelMessage;
+import org.apache.samza.sql.runner.SamzaSqlApplicationContext;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -45,38 +48,44 @@ import org.slf4j.LoggerFactory;
 class ProjectTranslator {
 
   private static final Logger LOG = LoggerFactory.getLogger(ProjectTranslator.class);
+  private final int queryId;
+  ProjectTranslator(int queryId) {
+    this.queryId = queryId;
+  }
 
-  void translate(final Project project, final TranslatorContext context) {
-    MessageStream<SamzaSqlRelMessage> messageStream = context.getMessageStream(project.getInput().getId());
-    List<Integer> flattenProjects =
-        project.getProjects().stream().filter(this::isFlatten).map(this::getProjectIndex).collect(Collectors.toList());
+  private static class ProjectMapFunction implements MapFunction<SamzaSqlRelMessage, SamzaSqlRelMessage> {
+    private transient Project project;
+    private transient Expression expr;
+    private transient TranslatorContext context;
 
-    if (flattenProjects.size() > 0) {
-      if (flattenProjects.size() > 1) {
-        String msg = "Multiple flatten operators in a single query is not supported";
-        LOG.error(msg);
-        throw new SamzaException(msg);
-      }
+    private final int queryId;
+    private final int projectId;
 
-      messageStream = translateFlatten(flattenProjects.get(0), messageStream);
+    ProjectMapFunction(int projectId, int queryId) {
+      this.projectId = projectId;
+      this.queryId = queryId;
     }
 
-    Expression expr = context.getExpressionCompiler().compile(project.getInputs(), project.getProjects());
+    @Override
+    public void init(Context context) {
+      this.context = ((SamzaSqlApplicationContext) context.getApplicationTaskContext()).getTranslatorContexts().get(queryId);
+      this.project = (Project) this.context.getRelNode(projectId);
+      this.expr = this.context.getExpressionCompiler().compile(project.getInputs(), project.getProjects());
+    }
 
-    MessageStream<SamzaSqlRelMessage> outputStream = messageStream.map(m -> {
+    @Override
+    public SamzaSqlRelMessage apply(SamzaSqlRelMessage message) {
       RelDataType type = project.getRowType();
       Object[] output = new Object[type.getFieldCount()];
       expr.execute(context.getExecutionContext(), context.getDataContext(),
-          m.getSamzaSqlRelRecord().getFieldValues().toArray(), output);
+          message.getSamzaSqlRelRecord().getFieldValues().toArray(), output);
       List<String> names = new ArrayList<>();
       for (int index = 0; index < output.length; index++) {
         names.add(index, project.getNamedProjects().get(index).getValue());
       }
 
       return new SamzaSqlRelMessage(names, Arrays.asList(output));
-    });
-
-    context.registerMessageStream(project.getId(), outputStream);
+    }
   }
 
   private MessageStream<SamzaSqlRelMessage> translateFlatten(Integer flattenIndex,
@@ -106,4 +115,27 @@ class ProjectTranslator {
   private Integer getProjectIndex(RexNode rexNode) {
     return ((RexInputRef) ((RexCall) rexNode).getOperands().get(0)).getIndex();
   }
+
+  void translate(final Project project, final TranslatorContext context) {
+    MessageStream<SamzaSqlRelMessage> messageStream = context.getMessageStream(project.getInput().getId());
+    List<Integer> flattenProjects =
+        project.getProjects().stream().filter(this::isFlatten).map(this::getProjectIndex).collect(Collectors.toList());
+
+    if (flattenProjects.size() > 0) {
+      if (flattenProjects.size() > 1) {
+        String msg = "Multiple flatten operators in a single query is not supported";
+        LOG.error(msg);
+        throw new SamzaException(msg);
+      }
+      messageStream = translateFlatten(flattenProjects.get(0), messageStream);
+    }
+
+    final int projectId = project.getId();
+
+    MessageStream<SamzaSqlRelMessage> outputStream = messageStream.map(new ProjectMapFunction(projectId, queryId));
+
+    context.registerMessageStream(project.getId(), outputStream);
+    context.registerRelNode(project.getId(), project);
+  }
+
 }
