@@ -44,8 +44,10 @@ import org.apache.samza.config.JobCoordinatorConfig;
 import org.apache.samza.config.MapConfig;
 import org.apache.samza.config.TaskConfigJava;
 import org.apache.samza.config.ZkConfig;
+import org.apache.samza.context.Context;
 import org.apache.samza.coordinator.JobCoordinator;
 import org.apache.samza.coordinator.JobCoordinatorFactory;
+import org.apache.samza.runtime.ProcessorLifecycleListener;
 import org.apache.samza.system.IncomingMessageEnvelope;
 import org.apache.samza.system.OutgoingMessageEnvelope;
 import org.apache.samza.system.SystemStream;
@@ -53,7 +55,6 @@ import org.apache.samza.task.InitableTask;
 import org.apache.samza.task.MessageCollector;
 import org.apache.samza.task.StreamTask;
 import org.apache.samza.task.StreamTaskFactory;
-import org.apache.samza.task.TaskContext;
 import org.apache.samza.task.TaskCoordinator;
 import org.apache.samza.test.StandaloneIntegrationTestHarness;
 import org.apache.samza.test.StandaloneTestUtils;
@@ -62,6 +63,7 @@ import org.apache.samza.zk.TestZkUtils;
 import org.apache.zookeeper.ZooKeeper;
 import org.apache.zookeeper.server.ZooKeeperServer;
 import org.junit.Assert;
+import org.junit.Before;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -90,7 +92,7 @@ public class TestZkStreamProcessorBase extends StandaloneIntegrationTestHarness
     return "";
   }
 
-//  @Before
+  @Before
   public void setUp() {
     super.setUp();
     // for each tests - make the common parts unique
@@ -132,13 +134,16 @@ public class TestZkStreamProcessorBase extends StandaloneIntegrationTestHarness
 
     Config config = new MapConfig(map);
     String jobCoordinatorFactoryClassName = new JobCoordinatorConfig(config).getJobCoordinatorFactoryClassName();
-    JobCoordinator jobCoordinator =
-        Util.getObj(jobCoordinatorFactoryClassName, JobCoordinatorFactory.class)
-            .getJobCoordinator(config);
+    JobCoordinator jobCoordinator = Util.getObj(jobCoordinatorFactoryClassName, JobCoordinatorFactory.class).getJobCoordinator(config);
 
-    StreamProcessorLifecycleListener listener = new StreamProcessorLifecycleListener() {
+    ProcessorLifecycleListener listener = new ProcessorLifecycleListener() {
       @Override
-      public void onStart() {
+      public void beforeStart() {
+
+      }
+
+      @Override
+      public void afterStart() {
         if (waitStart != null) {
             waitStart.countDown();
         }
@@ -146,16 +151,18 @@ public class TestZkStreamProcessorBase extends StandaloneIntegrationTestHarness
       }
 
       @Override
-      public void onShutdown() {
+      public void afterStop() {
+        // stopped w/o failure
         if (waitStop != null) {
           waitStop.countDown();
         }
-        LOG.info("onShutdown is called for pid=" + pId);
+        LOG.info("afterStop is called for pid=" + pId + " with successful shutdown");
       }
 
       @Override
-      public void onFailure(Throwable t) {
-        LOG.info("onFailure is called for pid=" + pId);
+      public void afterFailure(Throwable t) {
+        // stopped w/ failure
+        LOG.info("afterStop is called for pid=" + pId + " with failure");
       }
     };
 
@@ -333,8 +340,8 @@ public class TestZkStreamProcessorBase extends StandaloneIntegrationTestHarness
     protected String processorIdToFail;
 
     @Override
-    public void init(Config config, TaskContext taskContext)
-        throws Exception {
+    public void init(Context context) {
+      Config config = context.getJobContext().getConfig();
       this.processorId = config.get(ApplicationConfig.PROCESSOR_ID);
       this.outputTopic = config.get("app.outputTopic", "output");
       this.outputSystem = config.get("app.outputSystem", "test-system");
