@@ -21,9 +21,10 @@ package org.apache.samza.job.yarn
 import org.apache.hadoop.conf.Configuration
 import org.apache.hadoop.yarn.api.ApplicationConstants
 import org.apache.hadoop.yarn.api.records.ApplicationId
+import org.apache.samza.SamzaException
 import org.apache.samza.config.JobConfig.Config2Job
 import org.apache.samza.config.{Config, JobConfig, ShellCommandConfig, YarnConfig}
-import org.apache.samza.job.ApplicationStatus.{Running, SuccessfulFinish, UnsuccessfulFinish}
+import org.apache.samza.job.ApplicationStatus.{SuccessfulFinish, UnsuccessfulFinish}
 import org.apache.samza.job.{ApplicationStatus, StreamJob}
 import org.apache.samza.serializers.model.SamzaObjectMapper
 import org.apache.samza.util.{CoordinatorStreamUtil, Util}
@@ -66,7 +67,7 @@ class YarnJob(config: Config, hadoopConfig: Configuration) extends StreamJob {
           }
           envMapWithJavaHome
         }),
-        Some("%s_%s" format(config.getName.get, config.getJobId.getOrElse(1)))
+        Some("%s_%s" format(config.getName.get, config.getJobId))
       )
     } catch {
       case e: Throwable =>
@@ -115,7 +116,7 @@ class YarnJob(config: Config, hadoopConfig: Configuration) extends StreamJob {
       Thread.sleep(1000)
     }
 
-    Running
+    getStatus
   }
 
   def waitForStatus(status: ApplicationStatus, timeoutMs: Long): ApplicationStatus = {
@@ -130,14 +131,15 @@ class YarnJob(config: Config, hadoopConfig: Configuration) extends StreamJob {
       Thread.sleep(1000)
     }
 
-    Running
+    getStatus
   }
 
   def getStatus: ApplicationStatus = {
     getAppId match {
       case Some(appId) =>
         logger.info("Getting status for applicationId %s" format appId)
-        client.status(appId).getOrElse(null)
+        client.status(appId).getOrElse(
+          throw new SamzaException("No status was determined for applicationId %s" format appId))
       case None =>
         logger.info("Unable to report status because no applicationId could be found.")
         ApplicationStatus.SuccessfulFinish
@@ -167,7 +169,7 @@ class YarnJob(config: Config, hadoopConfig: Configuration) extends StreamJob {
         // Get by name
         config.getName match {
           case Some(jobName) =>
-            val applicationName = "%s_%s" format(jobName, config.getJobId.getOrElse(1))
+            val applicationName = "%s_%s" format(jobName, config.getJobId)
             logger.info("Fetching status from YARN for application name %s" format applicationName)
             val applicationIds = client.getActiveApplicationIds(applicationName)
 
